// xml.cc for raytance
//
// Started on  Thu Aug 22 13:17:43 2002
//  by Laurent CORBES {Caf'} caf@morkitu.com
// Last update Sat Dec 14 16:43:42 2002
//  by Laurent CORBES {Caf'} caf@morkitu.com
// Short Comment: xml loader based on libxml++ (need libxml 2)

#include <iostream>
#include <fstream>
#include "xml.hh"
#include "base.hh"
#include "types.hh"

extern int verbose;

namespace loadance
{
  LoadXml::LoadXml (const std::string& file)
    : _filename(file)
  { _scene=0; _ostr=0; _ti=0; _ci=0;};
  
  maintance::Scene* LoadXml::load ()
  {
    _tree= new XMLTree (_filename);
    if (_tree->initialized())
      {
	_scene = new maintance::Scene();
	treeRead(*(_tree->root()));
	delete _tree;
	
	// Comment: 
	// add a default camera if theres'not in the xmlfile
	//
	if (!_scene->GetCameraMap().size())
	  _scene->AddCamera(*new maintance::
			    Camera("freecam", maintance::Camera::FREE));
      }
    return _scene;
  }
  
  void LoadXml::convert (std::ofstream& ostr)
  {
    std::cout << "convert" << std::endl;
    load();
    _scene->print(std::cout);
    _ostr = &ostr;
    
    _tree= new XMLTree (_filename);
    if (_tree->initialized())
      {

	*_ostr << "maintance::Scene* _scene = new maintance::Scene();" 
	     << std::endl;
	
	treeRead_C(*(_tree->root()));
	delete _tree;
	
	// Comment: 
	// add a default camera if theres'not in the xmlfile
	//
	if (!_scene->GetCameraMap().size())
	  *_ostr << "_scene->AddCamera(*new maintance::Camera(\"freecam\", maintance::Camera::FREE));" << std::endl;
	*_ostr << "return _scene;" << std::endl;
      }
    
    return;
  }
  
  
  static const std::string environment_d = "Environment";
  static const std::string material_d = "Material";
  static const std::string sphere_d = "Sphere";
  static const std::string cylinder_d = "Cylinder";
  static const std::string box_d = "Box";
  static const std::string plane_d = "Plane";
  static const std::string disc_d = "Disc";
  static const std::string omni_d = "Omni";
  static const std::string camera_d = "Camera";
  static const std::string text_d = "Text";

  void LoadXml::treeRead (const XMLNode& node)
  {
    static int tab = 0;
    if (verbose)
      {
	for (int i = tab; i != 0; --i)
	  std::cout << "\t";
	std::cout <<  node.name() << std::endl;
	for (int i = tab; i != 0; --i)
	  std::cout << "\t";
	nodeRead(node);
      }
    ++tab;
    if (node.name() == environment_d)
      makeEnvironment(node);
    if (node.name() == material_d)
      makeMaterial(node);
    if (node.name() == sphere_d)
      makeSphere(node);
    if (node.name() == cylinder_d)
      makeCylinder(node);
    if (node.name() == box_d)
      makeBox(node);
    if (node.name() == plane_d)
      makePlane(node);
    if (node.name() == disc_d)
      makeDisc(node);
    if (node.name() == omni_d)
      makeOmni(node);
    if (node.name() == camera_d)
      makeCamera(node);
    if (node.name() == text_d)
      makeText(node);

    const XMLNodeList& children = node.children();
    for (XMLNodeConstIterator it = children.begin(); 
	 it != children.end(); ++it)
      {
	treeRead(**it);
      }
    --tab;
  };

  // convert tree reading funct
  void LoadXml::treeRead_C (const XMLNode& node)
  {
    if (node.name() == environment_d)
      makeEnvironment_C(node);
    if (node.name() == material_d)
      makeMaterial_C(node);
    if (node.name() == sphere_d)
      makeSphere_C(node);
    if (node.name() == cylinder_d)
      makeCylinder_C(node);
    if (node.name() == box_d)
      makeBox_C(node);
    if (node.name() == plane_d)
      makePlane_C(node);
    if (node.name() == disc_d)
      makeDisc_C(node);
    if (node.name() == omni_d)
      makeOmni_C(node);
    if (node.name() == camera_d)
      makeCamera_C(node);

    const XMLNodeList& children = node.children();
    for (XMLNodeConstIterator it = children.begin(); 
	 it != children.end(); ++it)
      {
	treeRead_C(**it);
      }
  };

  // sert a rien pour l'instant ;-)
  // affiche les proprietes d'un node
  void LoadXml::nodeRead (const XMLNode& node)
  {
    const XMLPropertyList& prop = node.properties();
    for (XMLPropertyConstIterator it = prop.begin();
	 it != prop.end(); ++it)
      {
	std::cout << (*it)->name() << "=" << (*it)->value() << ", ";
      }
    std::cout << std::endl;
  };

  // prise en compte de l'environement 
  void LoadXml::makeEnvironment (const XMLNode& env)
  {
    _scene->SetEnv(maintance::Environment(env.property("ambiant")->value(),
					  env.property("sky_horizon")->value(),
					  env.property("zenith")->value(),
					  env.property("ground_horizon")->value(),
					  env.property("ground")->value()
					  )
		   );
  };

  void LoadXml::makeEnvironment_C (const XMLNode& env)
  {
    *_ostr << "_scene->SetEnv(maintance::Environment(\"" 
	  << env.property("ambiant")->value() << "\",\""
	  << env.property("sky_horizon")->value() << "\",\""
	  << env.property("zenith")->value() << "\",\""
	  << env.property("ground_horizon")->value() << "\",\""
	  << env.property("ground")->value() << "\"));" 
	  << std::endl;
  };

  // ajout d'un shader dans la scene
  void LoadXml::makeMaterial (const XMLNode& mat)
  {
    _scene->
      AddShader(*new 
		maintance::Shader(mat.property("name")->value(), 
				  mat.property("diffuse")->value(), 
				  mat.property("specular_color")->value(),
				  atof(mat.property("opacity")->
				       value().c_str()),
				  atof(mat.property("reflexion")->
				       value().c_str()),
				  atof(mat.property("ior")->
				       value().c_str()),
				  atof(mat.property("specular_level")->
				       value().c_str()),
				  atof(mat.property("glossiness")->
				       value().c_str())
				  )
		);
  };
  void LoadXml::makeMaterial_C (const XMLNode& mat)
  {
    *_ostr << "_scene->AddShader(*new maintance::Shader(\""
	   << mat.property("name")->value() << "\",\""
	   << mat.property("diffuse")->value() << "\",\""
	   << mat.property("specular_color")->value() << "\","
	   << atof(mat.property("opacity")->value().c_str()) << ","
	   << atof(mat.property("reflexion")->value().c_str()) << ","
	   << atof(mat.property("ior")->value().c_str()) << ", "
	   << atof(mat.property("specular_level")->value().c_str()) << ", "
	   << atof(mat.property("glossiness")->value().c_str()) << "));"
	   << std::endl;
  };


  // ajout d'une sphere
  void LoadXml::makeSphere (const XMLNode& obj)
  {
    _scene->AddObject(*new 
		      maintance::Sphere(*(_scene->GetShader(obj.property("material")->value())),
					makeTransformer(obj),
					atof(obj.property("radius")->
					     value().c_str())
					)
		      );
  };
  void LoadXml::makeSphere_C (const XMLNode& obj)
  {
    makeTransformer_C(obj);
    *_ostr << "_scene->AddObject(*new maintance::Sphere(*(_scene->GetShader(\""
	   << obj.property("material")->value() << "\"))," 
	   << "T_" << _ti << ", "
	   << atof(obj.property("radius")->value().c_str()) << "));"
	   << std::endl;
  };

  void LoadXml::makeCylinder (const XMLNode& obj)
  {
    _scene->AddObject(*new 
		      maintance::Cylinder(*(_scene->GetShader(obj.property("material")->value())),
					  makeTransformer(obj),
					  types::Vect3df(
							 atof(obj.property("ox")->value().c_str()),
							 atof(obj.property("oy")->value().c_str()),
							 atof(obj.property("oz")->value().c_str()))
					  ,
					  atof(obj.property("height")->
					       value().c_str()),
					  atof(obj.property("radius")->
					       value().c_str())
					  )
		      );
  };

  void LoadXml::makeCylinder_C (const XMLNode& obj)
  {
    makeTransformer_C(obj);
    *_ostr << "_scene->AddObject(*new maintance::Cylinder(*(_scene->GetShader(\""
	   << obj.property("material")->value() << "\")),"
	   << "T_" << _ti << ", "
	   << "types::Vect3df("
	   << atof(obj.property("ox")->value().c_str()) << ","
	   << atof(obj.property("oy")->value().c_str()) << ","
	   << atof(obj.property("oz")->value().c_str()) << "),"
	   << atof(obj.property("height")->value().c_str()) << ","
	   << atof(obj.property("radius")->value().c_str()) << "));"
	   << std::endl;
  };

  void LoadXml::makeBox (const XMLNode& obj)
  {
    // voir si le vecteur est a partir de quel point ? orig

    types::Vect3df uln( atof(obj.property("width")->value().c_str()) / 2,
			 atof(obj.property("height")->value().c_str()) / 2,
			 atof(obj.property("length")->value().c_str()) / 2); 

    _scene->AddObject(*new 
		      maintance::Box(*(_scene->GetShader(obj.property("material")->value())),
				     makeTransformer(obj),
				     types::Vect3df(0,0,0) - uln,
				     uln
				     )
		      );
  };

  void LoadXml::makeBox_C (const XMLNode& obj)
  {
    // voir si le vecteur est a partir de quel point ? orig
    makeTransformer_C(obj);

    *_ostr << "types::Vect3df uln("
	   << atof(obj.property("width")->value().c_str()) / 2 << ","
	   << atof(obj.property("height")->value().c_str()) / 2 << ","
	   << atof(obj.property("length")->value().c_str()) / 2 << ");"
	   << std::endl
	   << "_scene->AddObject(*new maintance::Box(*(_scene->GetShader(\""
	   << obj.property("material")->value() << "\")),"
	   << "T_" << _ti << ", "
	   << "types::Vect3df(0,0,0) - uln, uln));"
	   << std::endl;
  };

  void LoadXml::makePlane (const XMLNode& obj)
  {
    _scene->AddObject(*new 
		      maintance::Plane(*(_scene->GetShader(obj.property("material")->value())),
				       makeTransformer(obj),
				       types::Vect3df(
						      atof(obj.property("vx")->value().c_str()),
						      atof(obj.property("vy")->value().c_str()),
						      atof(obj.property("vz")->value().c_str()))
				       )
		      );
  };

  void LoadXml::makePlane_C (const XMLNode& obj)
  {
    makeTransformer_C(obj);

    *_ostr << "_scene->AddObject(*new maintance::Plane(*(_scene->GetShader(\""
	   << obj.property("material")->value() << "\")),"
	   << "T_" << _ti << ", "
	   << "types::Vect3df("
	   << atof(obj.property("vx")->value().c_str()) << ","
	   << atof(obj.property("vy")->value().c_str()) << ","
	   << atof(obj.property("vz")->value().c_str()) << ")));"
	   << std::endl;
  };
  
  void LoadXml::makeDisc (const XMLNode& obj)
  {
    _scene->AddObject(*new 
		      maintance::Disc(*(_scene->GetShader(obj.property("material")->value())),
				      makeTransformer(obj),
				      types::Vect3df(
						     atof(obj.property("vx")->value().c_str()),
						     atof(obj.property("vy")->value().c_str()),
						     atof(obj.property("vz")->value().c_str())),
				      atof(obj.property("radius")->value().c_str())
				      )
		      );
  };

  void LoadXml::makeDisc_C (const XMLNode& obj)
  {
    makeTransformer_C(obj);
    *_ostr << "_scene->AddObject(*new maintance::Disc(*(_scene->GetShader(\""
	   << obj.property("material")->value() << "\")),"
	   << "T_" << _ti << ", "
	   << "types::Vect3df("
	   << atof(obj.property("vx")->value().c_str()) << ","
	   << atof(obj.property("vy")->value().c_str()) << ","
	   << atof(obj.property("vz")->value().c_str()) << "),"
	   << atof(obj.property("radius")->value().c_str()) << "));"
	   << std::endl;
  };
  

  void LoadXml::makeOmni(const XMLNode& omni)
  {
    _scene->AddLight(*new
		     maintance::Light(omni.property("name")->value(),
				      makeTransformer(omni),
				      maintance::Color(omni.property("color")->value())));
  };

  void LoadXml::makeOmni_C(const XMLNode& omni)
  {
    makeTransformer_C(omni),
    *_ostr << "_scene->AddLight(*new maintance::Light(\""
	   << omni.property("name")->value() << "\","
	   << "T_" << _ti << ", maintance::Color(\""
	   << omni.property("color")->value() << "\")));"
	   << std::endl;
  };

  maintance::Transformer& LoadXml::makeTransformer (const XMLNode& obj)
  {
    static const std::string trans_d = "Trans";
    static const std::string rot_d = "Rot";
    static const std::string scale_d = "Scale";
    
    maintance::Transformer &tr = *new maintance::Transformer();
    
    const XMLNodeList& children = obj.children();
    for (XMLNodeConstIterator it = children.begin();
	 it != children.end(); ++it)
      {
	if((*it)->name() == trans_d)
	  tr.AddTrans(types::Vect3df(atof((*it)
					  ->property("x")->value().c_str()), 
				     atof((*it)
					  ->property("y")->value().c_str()),
				     atof((*it)
					  ->property("z")->value().c_str())),
		      atof((*it)->property("frame")->value().c_str()));
  
	if((*it)->name() == rot_d)
	  tr.AddRot(types::Quatf(atof((*it)
					->property("w")->value().c_str()), 
				   atof((*it)
					->property("x")->value().c_str()), 
				   atof((*it)
					->property("y")->value().c_str()),
				   atof((*it)
					->property("z")->value().c_str())),
		      atof((*it)->property("frame")->value().c_str()));
	
	if((*it)->name() == scale_d)
	  tr.AddScale(types::Vect3df(atof((*it)
					  ->property("x")->value().c_str()), 
				     atof((*it)
					  ->property("y")->value().c_str()),
				     atof((*it)
					  ->property("z")->value().c_str())),
		      atof((*it)->property("frame")->value().c_str()));
      }
    return tr;
  };

  void LoadXml::makeTransformer_C (const XMLNode& obj)
  {
    _ti++;
    static const std::string trans_d = "Trans";
    static const std::string rot_d = "Rot";
    static const std::string scale_d = "Scale";
    
    *_ostr << "maintance::Transformer &T_"<< _ti << "= *new maintance::Transformer(); "
	   << std::endl;
    
    const XMLNodeList& children = obj.children();
    for (XMLNodeConstIterator it = children.begin();
	 it != children.end(); ++it)
      {
	if((*it)->name() == trans_d)
 	  *_ostr << "T_" << _ti << ".AddTrans(types::Vect3df("
		 << atof((*it)->property("x")->value().c_str()) << ", "
		 << atof((*it)->property("y")->value().c_str()) << ", "
		 << atof((*it)->property("z")->value().c_str()) << "), "
		 << atof((*it)->property("frame")->value().c_str()) << ");"
		 << std::endl;
	if((*it)->name() == rot_d)
	  *_ostr << "T_" << _ti << ".AddRot(types::Quatf("
		 << atof((*it)->property("w")->value().c_str()) << ", "
		 << atof((*it)->property("x")->value().c_str()) << ", " 
		 << atof((*it)->property("y")->value().c_str()) << ", "
		 << atof((*it)->property("z")->value().c_str()) << "), "
		 << atof((*it)->property("frame")->value().c_str()) << ");"
		 << std::endl;
	
	if((*it)->name() == scale_d)
	  *_ostr << "T_" << _ti << ".AddScale(types::Vect3df("
		 << atof((*it)->property("x")->value().c_str()) << ", "
		 << atof((*it)->property("y")->value().c_str()) << ", "
		 << atof((*it)->property("z")->value().c_str()) << "), "
		 << atof((*it)->property("frame")->value().c_str()) << ");"
		 << std::endl;
      }
    
  };

  void LoadXml::makeCamera(const XMLNode& camnode)
  {
    static const std::string center_d = "Center";
    static const std::string target_d = "Target";
    static const std::string fov_d = "Fov";
    
    maintance::Camera& cam = 
      *new maintance::Camera(camnode.property("name")->value());
    
    const XMLNodeList& children = camnode.children();
    for (XMLNodeConstIterator it = children.begin();
	 it != children.end(); ++it)
      {    
	if((*it)->name() == center_d)
	  cam.AddCenter(types::Vect3df(atof((*it)
					    ->property("x")->value().c_str()), 
				       atof((*it)
					    ->property("y")->value().c_str()),
				       atof((*it)
					    ->property("z")->value().c_str())),
			atof((*it)->property("frame")->value().c_str()));
	if((*it)->name() == target_d)
	  cam.AddTarget(types::Vect3df(atof((*it)
					    ->property("x")->value().c_str()), 
				       atof((*it)
					    ->property("y")->value().c_str()),
				       atof((*it)
					    ->property("z")->value().c_str())),
			atof((*it)->property("frame")->value().c_str()));
	if((*it)->name() == fov_d)
	  cam.AddFov(float(atof((*it)
				->property("z")->value().c_str())),
		     atof((*it)->property("frame")->value().c_str()));
      }
    _scene->AddCamera(cam);
  };
  void LoadXml::makeCamera_C(const XMLNode& camnode)
  {
    _ci++;
    static const std::string center_d = "Center";
    static const std::string target_d = "Target";
    static const std::string fov_d = "Fov";
    
    *_ostr << "maintance::Camera& cam_" << _ci <<"= *new maintance::Camera(\""
	   << camnode.property("name")->value() << "\");"
	   << std::endl;
    
    const XMLNodeList& children = camnode.children();
    for (XMLNodeConstIterator it = children.begin();
	 it != children.end(); ++it)
      {    
	if((*it)->name() == center_d)
	  *_ostr << "cam_" << _ci << ".AddCenter(types::Vect3df("
		 << atof((*it)->property("x")->value().c_str()) << ", " 
		 << atof((*it)->property("y")->value().c_str()) << ", "
		 << atof((*it)->property("z")->value().c_str()) << "), "
		 << atof((*it)->property("frame")->value().c_str()) << "); "
		 << std::endl;
	if((*it)->name() == target_d)
	  *_ostr << "cam_" << _ci << ".AddTarget(types::Vect3df("
		 << atof((*it)->property("x")->value().c_str()) << ", "
		 << atof((*it)->property("y")->value().c_str()) << ", "
		 << atof((*it)->property("z")->value().c_str()) << "), "
		 << atof((*it)->property("frame")->value().c_str()) << "); "
		 << std::endl;
	if((*it)->name() == fov_d)
	  *_ostr << "cam_" << _ci << ".AddFov(float("
		 << atof((*it)->property("z")->value().c_str()) << "),"
		 << atof((*it)->property("frame")->value().c_str()) <<");"
		 << std::endl;
      }
    *_ostr << "_scene->AddCamera(cam_" << _ci <<");" << std::endl;
  };

  void	LoadXml::makeText(const XMLNode& text)
  {
    maintance::Text& txt = *new maintance::Text(text.property("name")->value(),
						 text.property("color")->value());
    _scene->AddText(txt);
  };
}
