#ifndef _CAMERA_HH_
#define _CAMERA_HH_

#include <iostream>
#include <vector>
#include <string>
#include <cassert>
#include "types.hh"
#include "animkey.hh"

/*---------------------------------------------------------------------*\
|Camera animee composee d'un centre, d'une cible et d'une fov		|
|Fonctionnement similaire aux Transformer, SetTime pour animer		|
|la camera (si la camera est animee et non libre) et accesseurs		|
|classiques pour les attributs.						|
|La modification des attributs n'est valide que si la camera est libre	|
|(protection par des asserts).						|
\*---------------------------------------------------------------------*/


namespace maintance
{

	class Camera
	{
	public:
		static const int FREE = 0;
		static const int ANIMATE = 1;

		Camera(const std::string name, int kind = ANIMATE);
		Camera(const std::string name, types::Vect3df center,
			types::Vect3df target, 
			float fov);

	private:
		std::string _name;
		int _kind;
		float _fov;
		float _time;
		types::Vect3df _center;
		types::Vect3df _target;


		types::Curve<types::Vect3df> _center_curve;
		types::Curve<types::Vect3df> _target_curve;
		types::Curve<float> _fov_curve;

		void update();

	public:
		friend std::ostream& operator<<(std::ostream& ostr, 
			const Camera& cam)
		{
			cam.print(ostr);
			return ostr;
		}
		void	print(std::ostream& os) const;

		inline const std::string& getName() const
		{ return _name; }
		inline int const GetKind() const
		{return _kind;}
		inline const types::Vect3df& GetCenter() const
		{ return _center;}
		inline const types::Vect3df& GetTarget() const
		{ return _target;}
		inline const float& GetFov() const
		{ return _fov;}

		// Accesseurs pour modification, fonctionne seulement sur les freecam
		inline types::Vect3df& GetRCenter()
		{ assert(_kind == FREE); return _center;}
		inline types::Vect3df& GetRTarget()
		{ assert(_kind == FREE); return _target;}
		inline float& GetRFov()
		{ assert (_kind == FREE); return _fov;}


		inline void AddCenter(types::Vect3df center, float frame)
		{ assert(_kind == ANIMATE); 
		_center_curve.add(frame, center);};

		inline void AddTarget(types::Vect3df target, float frame)
		{ assert(_kind == ANIMATE); 
		_target_curve.add(frame, target);}

		inline void AddFov(float fov, float frame)
		{ assert(_kind == ANIMATE); 
		_fov_curve.add(frame, fov);}

		inline void SetTime(float time)
		{ assert (time >= 0); _time = time; update();}

	};



}


#endif // _CAMERA_HH_
