//	color.hh
//
//	Henry PRECHEUR - E-mail: henry.precheur@epita.fr
//	last modification Time-stamp: <20-Oct-2003 22:18:29 lcaf>

#ifndef	_COLOR_HH_
#define	_COLOR_HH_

#include <cassert>
#include <string>
#include <iostream>

namespace maintance
{
	class Color
	{
	public:
		Color();
		Color(float red,
			float green, 
			float blue);

		Color(const std::string str);


	public:
		enum ColorChanel
		{
			RED = 0,
			GREEN = 1,
			BLUE = 2,
		};

		const float*		GetColor() const;
		const float		       	getColor(ColorChanel cc) const;
		void			setColor(ColorChanel cc, unsigned char i);
		void			setColor(unsigned char red, 
			unsigned char green, 
			unsigned char blue);


		static inline void mult(Color& dst, const Color& a, const Color& b);
		static inline void mult(Color& dst, const Color& a, const Color& b, float c);
		static inline void mult(Color& dst, const Color& a, float c);
		static inline void mixe(Color& dst, const Color& a, const Color& b, float c);
		static inline float compare(const Color& c1, const Color& c2);

		Color operator+(const Color& rhs) const
		{
			Color tmp(_[0] + rhs._[0],
				_[1] + rhs._[1],
				_[2] + rhs._[2]);
			return tmp;
		}
		Color& operator+=(const Color& rhs)
		{
			_[0] += rhs._[0];
			_[1] += rhs._[1];
			_[2] += rhs._[2];
			return *this;
		}
		Color& operator+=(const float rhs)
		{
			_[0] += rhs;
			_[1] += rhs;
			_[2] += rhs;
			return *this;
		}


		Color operator-(const Color& rhs) const
		{
			Color tmp(_[0] - rhs._[0],
				_[1] - rhs._[1],
				_[2] - rhs._[2]);
			return tmp;
		}
		Color operator*(const Color& rhs) const
		{
			Color tmp((_[0] * rhs._[0]),
				(_[1] * rhs._[1]),
				(_[2] * rhs._[2]));
			return tmp;
		}
		Color operator+(const float& rhs) const
		{
			Color tmp((_[0] + rhs),
				(_[1] + rhs),
				(_[2] + rhs));
			return tmp;
		}
		Color operator-(const float& rhs) const
		{
			Color tmp((_[0] - rhs),
				(_[1] - rhs),
				(_[2] - rhs));
			return tmp;
		}
		Color operator*(const float& rhs) const
		{
			Color tmp((_[0] * rhs),
				(_[1] * rhs),
				(_[2] * rhs));
			return tmp;
		}

		bool	operator==(const Color& color) const
		{
			const float delta = 0.4f;

			for (unsigned int i = 0; i < 3; i++)
				if ((_[i] - color._[i]) > delta)
					return false;
			return true;
		}

		friend std::ostream&	operator<<(std::ostream& ostr,
			const Color& color)
		{
			ostr << "("
				<< (color.getColor(RED)) * 255 << " " 
				<< (color.getColor(GREEN)) * 255 << " " 
				<< (color.getColor(BLUE)) * 255 << ")";
			return ostr;
		};

	protected:

		float		_[3];



	};

	inline const float	Color::getColor(ColorChanel cc) const
	{
		assert((cc >= 0) && (cc <= 2));
		return (_[cc]);
	}
	inline void		Color::setColor(ColorChanel cc, unsigned char i)
	{
		_[cc] = i / 255.0f;
		assert((cc >= 0) && (cc <= 2));
		assert((i >= 0.0f) && (i <= 255.0f));
		_[cc] = i;
	}

	inline void		Color::setColor(unsigned char red, 
		unsigned char green, 
		unsigned char blue)
	{
		_[0] = red / 255.0f;
		_[1] = green / 255.0f;
		_[2] = blue / 255.0f;
	}

	inline const float*	Color::GetColor() const
	{
		return _;
	}


	inline void Color::mult(Color& dst, const Color& a, const Color& b)
	{
		dst._[0] = a._[0] * b._[0];
		dst._[1] = a._[1] * b._[1];
		dst._[2] = a._[2] * b._[2];
	}

	inline void Color::mult(Color& dst, const Color& a, const Color& b, float c)
	{
		dst._[0] = a._[0] * b._[0] * c;
		dst._[1] = a._[1] * b._[1] * c;
		dst._[2] = a._[2] * b._[2] * c;
	}

	inline void Color::mult(Color& dst, const Color& a, float c)
	{
		dst._[0] = a._[0] * c;
		dst._[1] = a._[1] * c;
		dst._[2] = a._[2] * c;
	}

	// Melange en 2 couleurs en fonction du coef c
	inline void Color::mixe(Color& dst, const Color& a, const Color& b, float c)
	{
		dst._[0] = a._[0] * (1.0f - c) +  c * b._[0];
		dst._[1] = a._[1] * (1.0f - c) +  c * b._[1];
		dst._[2] = a._[2] * (1.0f - c) +  c * b._[2];
	}

	// Retourne la diffrence entre 2 couleurs.
	inline float Color::compare(const Color& c1, const Color& c2)
	{
		float red = c1._[0] - c2._[0];
		float green = c1._[1] - c2._[1];
		float blue = c1._[2] - c2._[2];

		return (red > 0 ? red : -red) + (green > 0 ? green : -green) +(blue > 0 ? blue : -blue);
	}

}

#endif // _COLOR_HH_
