//	scene.hh
//
//	Henry PRECHEUR - E-mail: henry.precheur@epita.fr
//	last modification Time-stamp: <17-Sep-2002 11:27 by preche_h>

#ifndef	_SCENE_HH_
#define	_SCENE_HH_

#include <list>
#include <map>
#include <string>
#include <iostream>
#include <cassert>
#include "object.hh"
#include "light.hh"
#include "shader.hh"
#include "camera.hh"
#include "environment.hh"
//#include "text.hh"

namespace maintance
{
	typedef std::vector<Object*>	ObjectList;
	typedef std::vector<Light*>	LightList;
	typedef std::map<const std::string, Camera*>	CameraMap;
	typedef std::map<const std::string, Shader*>	ShaderMap;

	class Scene
	{
	public:
		Scene() 
		{
			_camera = 0;
			_env = Environment();
		};
		~Scene()
		{
			ShaderMap::iterator its;
			for (its = _shaderMap.begin(); its != _shaderMap.end(); ++its)
				delete its->second;
			CameraMap::iterator itc;
			for (itc = _cameraMap.begin(); itc != _cameraMap.end(); ++itc)
				delete itc->second;
			ObjectList::iterator ito;
			for (ito = _objList.begin(); ito != _objList.end(); ++ito)
				delete *ito;
			LightList::iterator itl;
			for (itl = _lightList.begin(); itl != _lightList.end(); ++itl)
				delete *itl;
		}

	private:
		ObjectList	_objList;
		LightList	_lightList;
		CameraMap	_cameraMap;
		ShaderMap	_shaderMap;

		// current camera
		Camera*	_camera;
		// the scene environment
		Environment _env;
		float	_time;

	public:
		void	AddObject(Object& obj);
		void	AddLight(Light& light);
		void	AddShader(Shader& shader);
		void	AddCamera(Camera& camera);
		const ObjectList&	GetObjList() const;
		const LightList&	GetLightList() const;
		const CameraMap&	GetCameraMap() const;
		const ShaderMap&	GetShaderMap() const;
		Shader*	GetShader(const std::string key);
		void	SetTime(const float t)
		{
			for(ObjectList::iterator it = _objList.begin();
				it != _objList.end(); ++it)
				(*it)->SetTime(t);
			for(LightList::iterator il = _lightList.begin();
				il != _lightList.end(); ++il)
				(*il)->SetTime(t);
			for(CameraMap::iterator itc = _cameraMap.begin();
				itc != _cameraMap.end(); ++itc)
				(*itc).second->SetTime(t);
			_time = t;
		};

		float	GetTime()
		{
			return _time;
		}
		void	SetCamera(const std::string cam)
		{
			_camera = _cameraMap.find(cam)->second;
			assert (_camera);
		};
		const Camera&	GetCamera() const
		{ return *_camera; };

		void	SetEnv(const Environment& env )
		{ _env = env; };
		const Environment&	GetEnv() const
		{return _env; };

		void print(std::ostream& os) const;
	};


	inline const ObjectList& Scene::GetObjList() const
	{ return _objList; }
	inline const LightList& Scene::GetLightList() const
	{ return _lightList; }
	inline const ShaderMap& Scene::GetShaderMap() const
	{ return _shaderMap; }
	inline const CameraMap& Scene::GetCameraMap() const
	{ return _cameraMap; }
	inline Shader* Scene::GetShader(const std::string key)
	{
		assert (_shaderMap.find(key) != _shaderMap.end());
		return _shaderMap[key];
	}
}

#endif // _SCENE_HH_
