//	shader.hh
//
//	Henry PRECHEUR - E-mail: henry.precheur@epita.fr
//	last modification Time-stamp: <20-Oct-2003 22:18:41 lcaf>

#ifndef	_SHADER_HH_
#define	_SHADER_HH_

#include <cassert>
#include <string>
#include "color.hh"

namespace maintance
{
	class Shader
	{
	public:
		Shader (const std::string& name, Color d = Color(1,1,1), 
			Color sc = Color(1,1,1), float o = 100.0f,
			float r = 0.0f, float i = 1.5f,
			float sl = 50.0f, float g = 10.0f)
			: _name(name), _diffuse(d), _specularC(sc)
		{ 
			setOpacity(o);
			setReflexion(r);
			setIor(i);
			setSpecularLevel(sl);
			setGlossiness(g);
		}
		Shader (const std::string& name, 
			const std::string d = "(color 100 100 100)",
			const std::string sc = "(color 100 100 100)",
			float o = 100.0f, float r = 0.0f,
			float i = 1.5f, float sl = 50.0f, float g = 10.0f)
			: _name(name), _diffuse(d), _specularC(sc)
		{
			setOpacity(o);
			setReflexion(r);
			setIor(i);
			setSpecularLevel(sl);
			setGlossiness(g);
		}

	public:
		friend std::ostream& operator<<(std::ostream& ostr, 
			const Shader& shad)
		{
			shad.print(ostr);
			return ostr;
		}
		void	print(std::ostream& os) const;

	public:
		const std::string&	getName() const;
		const Color&	getDiffuse() const;
		const Color&	getSpecularColor() const;
		const float&	getOpacity() const;
		const float&	getReflexion() const;
		const float&	getIor() const;
		const float&	getSpecularLevel() const;
		const float&	getGlossiness() const;
		void	setDiffuse(Color& color);
		void	setSpecularColor(Color& color);
		void	setOpacity(float f);
		void	setReflexion(float f);
		void	setIor(float f);
		void	setSpecularLevel(float f);
		void	setGlossiness(float f);

	private:
		const std::string	_name;
		Color	_diffuse;
		Color	_specularC;
		float	_opacity;
		float	_reflexion;
		float	_ior;
		float	_specularL;
		float	_glossiness;
	};

	inline const std::string& Shader::getName() const
	{ return _name; }
	inline const Color& Shader::getDiffuse() const
	{ return _diffuse; }
	inline const Color& Shader::getSpecularColor() const
	{ return _specularC; }
	inline const float&	Shader::getOpacity() const
	{ return _opacity; }
	inline const float&	Shader::getReflexion() const
	{ return _reflexion; }
	inline const float&	Shader::getIor() const
	{ return _ior; }
	inline const float&	Shader::getSpecularLevel() const
	{ return _specularL; }
	inline const float&	Shader::getGlossiness() const
	{ return _glossiness; }

	inline void	Shader::setDiffuse(Color& color)
	{ _diffuse = color; }
	inline void	Shader::setSpecularColor(Color& color)
	{ _specularC = color; }
	inline void	Shader::setOpacity(float f)
	{ 
		assert((f >= 0.0f) && (f <= 100.0f));
		_opacity = f;
	}
	inline void	Shader::setReflexion(float f)
	{ 
		assert((f >= 0.0f) && (f <= 100.0f));
		_reflexion = f / 100.0f;
	}
	inline void	Shader::setIor(float f)
	{ 
		assert((f >= 0.0f) && (f <= 100.0f)); //FIXME
		_ior = f;
	}
	inline void	Shader::setSpecularLevel(float f)
	{ 
		assert((f >= 0.0f) && (f <= 100.0f));
		_specularL = f / 20.0f;
	}
	inline void	Shader::setGlossiness(float f)
	{ 
		assert((f >= 0.0f) && (f <= 100.0f));
		_glossiness = f / 5.0f;
	}
}

#endif // _SHADER_HH_
