#ifndef _MAINTANCE_TRANSFORMER_HH_
#define _MAINTANCE_TRANSFORMER_HH_

#include <vector>
#include <cassert>
#include "types.hh"
#include "position.hh"
#include "animkey.hh"

/*---------------------------------------------------------------*\
La classe Transformer est utilisee pour placer les objets dans
l'espace (Trans + Rot + Scale) en gerant l'animation. A partir 
de courbe d'animation, on defini d'abord le temp courant (SetTime) 
puis on recupere la position courante (GetPos).
\*---------------------------------------------------------------*/


namespace maintance
{
	class Transformer {

	public:
		Transformer();
		virtual ~Transformer();
	private:

		float _time;
		Position _pos;

		types::Curve<types::Vect3df> _trans_curve;
		types::Curve<types::Vect3df> _scale_curve;

		void update();
/*		virtual types::Quatf interpol_quat(unsigned int index, 
			float h, 
			types::RtVector<QuatKey * > curve);
*/
	public:

		inline float Transformer::GetTime() const;
		inline void Transformer::SetTime(float time);
		inline const Position& GetPos() const;

		void AddTrans(types::Vect3df trans, float frame);
		void AddScale(types::Vect3df scale, float frame);

	};


	inline float Transformer::GetTime() const
	{ return _time ;}
	inline void Transformer::SetTime(float time)
	{ assert(time >= 0); _time = time; update(); }
	inline const Position& Transformer::GetPos() const
	{ return _pos;}
}

#endif // _MAINTANCE_TRANSFORMER_HH_
