#include "raytracer.hh"

namespace raytrace 
{
	void Raytracer::init_RtObjs()
	{
		_zenith.setColor(200, 200, 240);
		_horizon.setColor(200, 200, 200);
		_nadir.setColor(200, 200, 200);
		_ground.setColor(255, 255, 255);

		//_obj_vect = new RtObject*[_scene.GetObjList().size()];

		_sphere_list = new RtSphere*[MAX_NB_OBJ];
		_plane_list = new RtPlane*[MAX_NB_OBJ];
		_box_list = new RtBox*[MAX_NB_OBJ];
		_disc_list = new RtDisc*[MAX_NB_OBJ];
		_cylinder_list = new RtCylinder*[MAX_NB_OBJ];


		for (unsigned int i = 0; i < MAX_NB_OBJ; i++)
		{
			_sphere_list[i] = NULL;
			_plane_list[i] = NULL;
			_box_list[i] = NULL;
			_disc_list[i] = NULL;
			_cylinder_list[i] = NULL;
		}

		for(unsigned int i = 0; i < _scene.GetObjList().size(); i++)
			_rtobjfact.AddObj(_sphere_list, 
			_plane_list,
			_box_list,
			_disc_list,
			_cylinder_list,
			(_scene.GetObjList())[i], MAX_NB_OBJ);


		_obj_light = new RtObject*[_scene.GetLightList().size()];
		for(unsigned int i = 0; i < _scene.GetLightList().size(); i++)
			_obj_light[i] = 0;

	}

	void Raytracer::Update()
	{
		_cam.update();

		__ = rand();
		for (unsigned int i = 0; _sphere_list[i]; ++i)
			_sphere_list[i]->update();
		for (unsigned int i = 0; _plane_list[i]; ++i)
			_plane_list[i]->update();
		for (unsigned int i = 0; _box_list[i]; ++i)
			_box_list[i]->update(); 
		for (unsigned int i = 0; _cylinder_list[i]; ++i)
			_cylinder_list[i]->update();
	}

	void Raytracer::RenderPixel(systance::Pixel* pixel)
	{
		static Ray rcur;
		static Ray rcur2;

		rcur = _cam.eval_ray(pixel->_PosX, pixel->_PosY);
		//rcur2 = _cam.eval_ray(pixel->_PosX + cos(pixel->_PosY / 5) * 4, pixel->_PosY);
		rcur2 = rcur;
		float z = -1;
		float zt;
		RtObject* last = 0;

		// Collision avec tous les types d'objets
		for (unsigned int i = 0; _sphere_list[i]; ++i)
			if ((zt = _sphere_list[i]->primaryHit(rcur2)) > 0 && (zt < z || z == -1))
			{
				last = _sphere_list[i];
				z = zt;
			}

			for (unsigned int i = 0; _plane_list[i]; ++i)
				if ((zt = _plane_list[i]->primaryHit(rcur)) > 0 && (zt < z || z == -1))
				{
					last = _plane_list[i];
					z = zt;
				}

				for (unsigned int i = 0; _box_list[i]; ++i)
					if ((zt = _box_list[i]->primaryHit(rcur)) > 0 && (zt < z || z == -1))
					{
						last = _box_list[i];
						z = zt;
					}

					for (unsigned int i = 0; _disc_list[i]; ++i)
						if ((zt = _disc_list[i]->primaryHit(rcur)) > 0 && (zt < z || z == -1))
						{
							last = _disc_list[i];
							z = zt;
						}

						for (unsigned int i = 0; _cylinder_list[i]; ++i)
							if ((zt = _cylinder_list[i]->primaryHit(rcur)) > 0 && (zt < z || z == -1))
							{
								last = _cylinder_list[i];
								z = zt;
							}

							pixel->_PosZ = z;

							if (last)
							{
								pixel->ReflexId = 0;
								EvalPx(pixel, *last, rcur, z, 0);
								pixel->ObjId = (*last).GetId();
								EvalFog(true, pixel, rcur, z);
							}
							else
							{
								pixel->ObjId = EvalBG(pixel, rcur);
								EvalFog(false, pixel, rcur, z);
								pixel->_UV._[0] = pixel->_UV._[1] = 0;
								pixel->ShadowId = 0;
								pixel->ReflexId = 0;
							}
	}

	void Raytracer::EvalPx(systance::Pixel* pix,
		RtObject& obj,
		Ray& ray,
		float z,
		unsigned int depth)
	{

		static types::Vect3df vtemp;
		static types::Vect3df pt;
		static maintance::Color spec;
		static maintance::Color ctemp;
		static Ray shadow;
		static Ray reflect;

		float light_dist;
		float sh_tmp = 0;
		float i,j,soft_shadow, soft_dist = sh_tmp;

		int li = 0;
		types::Vect3df::addmult(pt, ray.pos, ray.dest, obj.GetT());
		//    pt = ray.pos + ray.dest * obj.GetT();
		shadow.pos = reflect.pos = pt;
		const types::Vect3df& norm(obj.GetNorm(pt));

		pix->_Color.setColor(10, 20, 20);
		pix->ShadowId = 0;

		spec.setColor(0, 0, 0);


		// Calcul du rayon reflechi
		// reflect.dest = ray.dest - norm * 2 * norm.dot(ray.dest);
		types::Vect3df::submult(reflect.dest, ray.dest, norm, 2 * norm.dot(ray.dest));
		reflect.dest.normalise();


		// Pour toutes les lumieres
		for (std::vector<maintance::Light*>::const_iterator k = 
			_scene.GetLightList().begin();
			k != _scene.GetLightList().end();
		++k, ++li)
		{
			// Calcul du rayon partant du point d'intersection vers la lumiere
			types::Vect3df::sub(vtemp, (*k)->GetTransformer().GetPos().GetTrans(), pt);
			light_dist = vtemp.getNorm();
			shadow.dest = vtemp;
			shadow.dest /= light_dist;

			i = shadow.dest.dot(norm) * 2;
			soft_shadow = 1;

			// Calcul des ombres portees
			if (obj.receive_shadow && _obj_light[li] && _obj_light[li] != &obj && (sh_tmp = _obj_light[li]->secondaryHit(shadow)) > 0 && (sh_tmp < light_dist))
			{
				soft_shadow = -1;
				soft_dist = sh_tmp;
			}
			if (i > 0 && obj.receive_shadow)
			{
				for(unsigned int idx = 0; _sphere_list[idx]; ++idx)
				{
					if (_sphere_list[idx] != &obj && _sphere_list[idx]->cast_shadow && (_sphere_list[idx] != _obj_light[li])
						&& (sh_tmp = _sphere_list[idx]->secondaryHit(shadow)) > 0 && (sh_tmp < light_dist))
					{
						_obj_light[li] = _sphere_list[idx];
						soft_shadow = -1;
						soft_dist = sh_tmp;
						break;
					}
				}
				if (soft_shadow == 1)
				{
					for(unsigned int idx = 0; _box_list[idx]; ++idx)
					{
						if (_box_list[idx] != &obj && _box_list[idx]->cast_shadow && (_box_list[idx] != _obj_light[li])
							&& (sh_tmp = _box_list[idx]->secondaryHit(shadow)) > 0 && (sh_tmp < light_dist))
						{
							_obj_light[li] = _box_list[idx];
							soft_shadow = -1; 
							soft_dist = sh_tmp;
							break;
						}
					}
					if (soft_shadow == 1)
					{
						for(unsigned int idx = 0; _disc_list[idx]; ++idx)
						{
							if (_disc_list[idx] != &obj && _disc_list[idx]->cast_shadow && (_disc_list[idx] != _obj_light[li])
								&& (sh_tmp = _disc_list[idx]->secondaryHit(shadow)) > 0 && (sh_tmp < light_dist))
							{
								_obj_light[li] = _disc_list[idx];
								soft_shadow = -1; 
								soft_dist = sh_tmp;
								break;
							}
						}
						if (soft_shadow == 1)
						{
							for(unsigned int idx = 0; _cylinder_list[idx]; ++idx)
							{
								if (_cylinder_list[idx] != &obj && _cylinder_list[idx]->cast_shadow && (_cylinder_list[idx] != _obj_light[li])
									&& (sh_tmp = _cylinder_list[idx]->secondaryHit(shadow)) > 0 && (sh_tmp < light_dist))
								{
									_obj_light[li] = _cylinder_list[idx];
									soft_shadow = -1; 
									soft_dist = sh_tmp;
									break;
								}
							}
						}
					}
				}
			}
			if (soft_shadow == -1)
			{
				pix->ShadowId += 1;
				if (soft_dist > _max_ldist)
					soft_shadow = 1;
				else
				{
					//soft_shadow = 0;
					soft_shadow = soft_dist / _max_ldist;
				}
			}
			// Si pas d'ombre
			i *= soft_shadow;	    
			if (i > 0)
			{

				// Calcul du Specular
				j = 1;
				float tmp2 = shadow.dest.dot(reflect.dest);
				int signe = tmp2 >= 0 ? 1 : -1;
				tmp2 = tmp2 >= 0 ? tmp2 : - tmp2;
				for (int tmp = 0; tmp <= obj.GetShader()->getGlossiness();
					tmp++)
					j *= tmp2;
				j *= signe;

				// Application du diffuse
				maintance::Color::mult(ctemp, obj.GetShader()->getDiffuse(), (*k)->GetColor(), i);
				pix->_Color += ctemp;
				//pix->_Color += obj.GetShader()->getDiffuse() * 
				//  (*k)->GetColor() * i * 2;

				// Application du specular
				j *= soft_shadow;
				if (j > 0)
				{
					maintance::Color::mult(ctemp, (*k)->GetColor(),  obj.GetShader()->getSpecularLevel() * j);
					spec += ctemp;
					pix->_Color += spec;
					// spec += (*k)->GetColor() * 
					//obj.GetShader()->getSpecularLevel() * j;

				}
			}
			pix->ShadowId <<= 1;
		}

		// Calcul des reflexions
		if (obj.GetShader()->getReflexion() && depth <= _max_rec)
		{
			reflect.pos = pt;
			EvalRefRay(depth, obj, reflect, &_pixel_stack[depth]);
			maintance::Color::mixe(pix->_Color, pix->_Color, _pixel_stack[depth]._Color, obj.GetShader()->getReflexion());

			//pix->_Color = pix->_Color * (1.0 - obj.GetShader()->getReflexion()) +
			//  ref._Color * obj.GetShader()->getReflexion();
			pix->ReflexId = _pixel_stack[depth].ReflexId;
			pix->_UV._[0] = _pixel_stack[depth]._UV._[0];
			pix->_UV._[1] = _pixel_stack[depth]._UV._[1];
			pix->ShadowId <<= 16;
			pix->ShadowId += _pixel_stack[depth].ShadowId;
		}
		else
			obj.SetUV(pt, pix->_UV._[0], pix->_UV._[1]);
	}

	void Raytracer::EvalRefRay(unsigned int depth, RtObject& obj, Ray& ray,
		systance::Pixel* ref)
	{
		float z = -1;
		float zt;
		RtObject* last = 0;

		// Collision avec tous les types d'objets
		for (unsigned int i = 0; _sphere_list[i]; ++i)
			if ((_sphere_list[i] != &obj) && (zt = _sphere_list[i]->secondaryHit(ray)) > 0 && (zt < z || z == -1))
			{
				last = _sphere_list[i];
				z = zt;
			}

			for (unsigned int i = 0; _plane_list[i]; ++i)
				if ((_plane_list[i] != &obj) && (zt = _plane_list[i]->secondaryHit(ray)) > 0 && (zt < z || z == -1))
				{
					last = _plane_list[i];
					z = zt;
				}

				for (unsigned int i = 0; _box_list[i]; ++i)
					if ((_box_list[i] != &obj) && (zt = _box_list[i]->secondaryHit(ray)) > 0 && (zt < z || z == -1))
					{
						last = _box_list[i];
						z = zt;
					}

					for (unsigned int i = 0; _disc_list[i]; ++i)
						if ((_disc_list[i] != &obj) && (zt = _disc_list[i]->secondaryHit(ray)) > 0 && (zt < z || z == -1))
						{
							last = _disc_list[i];
							z = zt;
						}

						for (unsigned int i = 0; _cylinder_list[i]; ++i)
							if ((_cylinder_list[i] != &obj) && (zt = _cylinder_list[i]->secondaryHit(ray)) > 0 && (zt < z || z == -1))
							{
								last = _cylinder_list[i];
								z = zt;
							}

							if (last)
							{
								EvalPx(ref, *last, ray, z, depth + 1);
								ref->ReflexId = (*last).GetId();
							}
							else
							{
								ref->ReflexId = EvalBG(ref, ray);
								ref->_UV._[0] = ref->_UV._[1] = 0;
							}
	}

	int Raytracer::EvalBG(systance::Pixel* pix, Ray& ray)
	{
		static types::Vect3df temp(0, 1, 0);

		float i = temp.dot(ray.dest);

		if (i >= 0)
		{
			maintance::Color::mixe(pix->_Color, _horizon, _zenith, i);
			//	pix->_Color = _horizon * (1 - i) + _zenith * i;
			return 0;
		}
		else
		{
			i *= -1;
			//	pix->_Color = _nadir * (1 - i) + _ground * i;
			maintance::Color::mixe(pix->_Color, _nadir, _ground, i);
			return  0xffff - 1;
		}
	}


	void Raytracer::EvalFog(bool hit, systance::Pixel* pix, Ray& ray, float z)
	{

		float lf_near = 0;
		float lf_far = 0;
		float tmp;
		// calcul du brouillard en couche
		if (_l_fog)
		{
			// Rayon parallele a l'horizontal
			if (ray.dest.gety() == 0)
			{
				if (ray.pos.gety() > _lf_top || ray.pos.gety() < _lf_bottom)
				{
					lf_near = 0;
					lf_far = 0;
				}
				else
				{
					lf_near = 0;
					lf_far = _lf_top - _lf_bottom;
				}
			} 
			else
			{
				// Si le rayon est hors du brouillard
				if (ray.pos.gety() > _lf_top || ray.pos.gety() < _lf_bottom)
				{
					// Calcul du pres et du loin
					lf_near = (_lf_top - ray.pos.gety()) / ray.dest.gety();
					lf_far = (_lf_bottom - ray.pos.gety()) / ray.dest.gety();
					if (lf_near < 0)
						lf_near = 0;
					if (lf_far < 0)
						lf_far = 0;
					if (lf_far < lf_near)
					{
						tmp = lf_far;
						lf_far = lf_near;
						lf_near = tmp;
					}
				}
				else
				{
					lf_near = 0;
					// Calcul du loin
					if (ray.dest.gety() > 0)
						lf_far = (_lf_top - ray.pos.gety()) / ray.dest.gety();
					else
						lf_far = (_lf_bottom - ray.pos.gety()) / ray.dest.gety();
				}
				if (hit)
				{
					if (z < lf_near)
					{
						return;
					}
					if (z < lf_far)
					{

						lf_far = z;
					}
				}

				tmp = (lf_far - lf_near) / (_lf_top - _lf_bottom);
				if (tmp > 1)
					tmp = 1;
				tmp *= _lf_density;
				maintance::Color::mixe(pix->_Color, pix->_Color, _lf_color, tmp);
			}
			// calcul du brouillard en distance
			if (_fog)
			{
				if (z >= _far)
					z = _far;
				if (z >= _near)
				{
					z = (z - _near) / (_far - _near) * _cfar;
					maintance::Color::mixe(pix->_Color, pix->_Color, _fogcolor, z);
				}    
			}
		}
	}

}
