// raytracer.hh for raytance
//
// Started on  Fri Aug 23 18:21:11 2002
//  by Laurent CORBES {Caf'} caf@morkitu.com
// Last update Sat Dec 14 06:56:53 2002
//  by Laurent CORBES {Caf'} caf@morkitu.com
// Short Comment: the main raytracer class

#ifndef _RAYTRACE_RAYTRACER_HH_
#define _RAYTRACE_RAYTRACER_HH_

#include "scene.hh"
#include "rtcam.hh"
#include "rtobj.hh"
#include "systance/pixel.hh"
#include <vector>
#include "rtobj.hh"
#include "rtobjfactory.hh"

namespace raytrace
{
	class Raytracer
	{

	public:

		Raytracer(maintance::Scene& scene, unsigned width, unsigned height)
			:_scene(scene),
			_cam(&scene.GetCamera(), width, height),
			_width(width), 
			_height(height),
			_max_rec(MAX_REC),
			_fog(false)
		{ 
			// Temporaire, a passer dans le fichier XML.
			_fog = false;
			_fogcolor.setColor(0, 0, 0);
			_near = 2;
			_far = 4;
			_cfar = 1;

			_l_fog = true;
			_lf_color.setColor(200, 200, 200);
			_lf_bottom = 0;
			_lf_top = 0.5f;
			_lf_density = 0.8f;
			init_RtObjs();

			_max_ldist = 3;
			_pixel_stack = new systance::Pixel[_max_rec];
		}

		void Update();
		void RenderPixel(systance::Pixel* pixel);
		void setMaxRec(unsigned int max)
		{	_max_rec = max; }


		void EvalPx(systance::Pixel*	pix, 
			RtObject&		obj, 
			Ray&			ray,
			float			z,
			unsigned int		depth);

		void EvalRefRay(unsigned int	depth, 
			RtObject&		obj, 
			Ray&		ray,
			systance::Pixel*	ref);    
		int EvalBG(systance::Pixel*	pix, Ray& ray);

		void EvalFog(bool hit, systance::Pixel* pix, Ray& ray, float z);
	private:


		static const unsigned int MAX_REC = 2;
		static const unsigned int MAX_NB_OBJ = 20;

	protected:


		void init_RtObjs();

		RtObjFactory		_rtobjfact;
		maintance::Scene&		_scene;


		// Ciel et Sol.
		maintance::Color		_zenith;
		maintance::Color		_horizon;
		maintance::Color		_ground;
		maintance::Color		_nadir;

		RtCam			_cam;
		RtObject**			_obj_vect;
		RtSphere**			_sphere_list;
		RtPlane**			_plane_list;
		RtBox**			_box_list;
		RtDisc**			_disc_list;
		RtCylinder**		_cylinder_list;

		unsigned int		_nb_obj;
		RtObject**			_obj_light;
		unsigned int		_width;
		unsigned int		_height;
		unsigned int		_max_rec;

		systance::Pixel*		_pixel_stack;

		// Paramtres du brouillard

		bool		_fog;
		maintance::Color	_fogcolor;
		float		_near;
		float		_far;
		float		_cfar;


		// Brouillard en couche
		bool		_l_fog;
		maintance::Color	_lf_color;
		float		_lf_top;
		float		_lf_bottom;
		float		_lf_density;
		int			__; // very private
		float		_max_ldist;

	public:

		// Couleurs du ciel
		void setZenith(const maintance::Color& col)
		{ _zenith = col;}
		void setHorizon(const maintance::Color& col)
		{ _horizon = col;}
		void setGround(const maintance::Color& col)
		{ _ground = col;}
		void setNadir(const maintance::Color& col)
		{ _nadir = col;}

		// brouillard;
		void setFog(bool fog)
		{ _fog = fog;}
		void setFogColor(const maintance::Color& col)
		{ _fogcolor = col;}
		/*void setFogNear(float near)
		{ _near = near;}
		void setFogFar(float far)
		{ _far = far;}*/
		void setFogCFar(float cfar)
		{ _cfar = cfar;}

	};
}

#endif // _RAYTRACE_RAYTRACER_HH_
