#ifndef _RTBOX_HH_
#define _RTBOX_HH_

#include "rtobj.hh"
#include "raytracer.hh"
#include "box.hh"

namespace raytrace
{

	class Raytracer;

	class RtBox : public RtObject
	{
		typedef struct f_box
		{
			types::Vect3df pt;
			types::Vect3df norm;
			float u, v;
			float dist;
		}face_box;



	public:
		RtBox (maintance::Box& obj)
			: RtObject(obj), _box(obj)
		{
			_id_save = _id;
			faces[0].norm.set(-1, 0, 0);
			faces[1].norm.set(0, 0, -1);
			faces[2].norm.set(0, -1, 0);
			faces[3].norm.set(0, 0, 1);
			faces[4].norm.set(1, 0, 0);
			faces[5].norm.set(0, 1, 0);
		}

		virtual ~RtBox()
		{}

		void			faces_update(Ray& ray)
		{
			static types::Vect3df delta;
			static types::Vect3df uln;
			static types::Vect3df lrf;

			delta = _box.GetLowerRightFar() - _box.GetUpperLeftNear();
			delta.setx(delta.getx() * _pos.GetScale().getx());
			delta.sety(delta.gety() * _pos.GetScale().gety());
			delta.setz(delta.getz() * _pos.GetScale().getz());


			uln = _box.GetUpperLeftNear();
			uln.setx(uln.getx() * _pos.GetScale().getx());
			uln.sety(uln.gety() * _pos.GetScale().gety());
			uln.setz(uln.getz() * _pos.GetScale().getz());
			uln += _pos.GetTrans();

			lrf = _box.GetLowerRightFar();
			lrf.setx(lrf.getx() * _pos.GetScale().getx());
			lrf.sety(lrf.gety() * _pos.GetScale().gety());
			lrf.setz(lrf.getz() * _pos.GetScale().getz());
			lrf += _pos.GetTrans();


			faces[0].pt = uln;
			faces[0].u = delta.getz();
			faces[0].v = delta.gety();
			faces[0].dist = faces[0].pt.getx() - ray.pos.getx();

			faces[1].pt = uln;
			faces[1].u = delta.getx();
			faces[1].v = delta.gety();
			faces[1].dist = faces[1].pt.getz() - ray.pos.getz();

			faces[2].pt = uln;
			faces[2].u = delta.getx();
			faces[2].v = delta.getz(); 
			faces[2].dist = faces[2].pt.gety() - ray.pos.gety();

			delta *= -1;

			faces[3].pt = lrf;
			faces[3].v = delta.gety();
			faces[3].u = delta.getx();
			faces[3].dist = faces[3].pt.getz() - ray.pos.getz();

			faces[4].pt = lrf;
			faces[4].v = delta.gety();
			faces[4].u = delta.getz();
			faces[4].dist = faces[4].pt.getx() - ray.pos.getx();

			faces[5].pt = lrf;
			faces[5].v = delta.getz();
			faces[5].u = delta.getx();
			faces[5].dist = faces[5].pt.gety() - ray.pos.gety();
		}


		virtual float		primaryHit(Ray& ray)
		{
			float t;
			static types::Vect3df temp;

			if (_update)
			{
				faces_update(ray);
				_update = false;
			}

			if (ray.dest.dot(faces[0].norm) < 0 && ray.dest.getx())
			{	
				t = faces[0].dist / ray.dest.getx();
				temp = (ray.pos + ray.dest * t) - faces[0].pt;
				if ( temp.gety() > 0 && temp.getz() > 0 && 
					temp.gety() < faces[0].v && temp.getz() < faces[0].u)
				{
					_face_id = 0;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[1].norm) < 0 && ray.dest.getz())
			{	
				t = (faces[1].pt.getz() - ray.pos.getz()) / ray.dest.getz();
				temp = (ray.pos + ray.dest * t) - faces[1].pt;
				if ( temp.gety() > 0 && temp.getx() > 0 && 
					temp.gety() < faces[1].v && temp.getx() < faces[1].u)
				{
					_face_id = 1;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[2].norm) < 0 && ray.dest.gety())
			{	
				t = (faces[2].pt.gety() - ray.pos.gety()) / ray.dest.gety();
				temp = (ray.pos + ray.dest * t) - faces[2].pt;
				if ( temp.getx() > 0 && temp.getz() > 0 && 
					temp.getx() < faces[2].u && temp.getz() < faces[2].v)
				{
					_face_id = 2;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[3].norm) < 0 && ray.dest.getz())
			{	
				t = (faces[3].pt.getz() - ray.pos.getz()) / ray.dest.getz();
				temp = (ray.pos + ray.dest * t) - faces[3].pt;
				if ( temp.getx() < 0 && temp.gety() < 0 && 
					temp.getx() > faces[3].u && temp.gety() > faces[3].v)
				{
					_face_id = 3;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[4].norm) < 0 && ray.dest.getx())
			{	
				t = (faces[4].pt.getx() - ray.pos.getx()) / ray.dest.getx();
				temp = (ray.pos + ray.dest * t) - faces[4].pt;
				if ( temp.getz() < 0 && temp.gety() < 0 && 
					temp.getz() > faces[4].u && temp.gety() > faces[4].v)
				{
					_face_id = 4;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[5].norm) < 0 && ray.dest.gety())
			{	
				t = (faces[4].pt.gety() - ray.pos.gety()) / ray.dest.gety();
				temp = (ray.pos + ray.dest * t) - faces[5].pt;
				if ( temp.getz() < 0 && temp.getx() < 0 && 
					temp.getx() > faces[5].u && temp.getz() > faces[5].v)
				{
					_face_id = 5;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			return -1;
		}



		virtual float		secondaryHit(Ray& ray)
		{
			float t;
			static types::Vect3df temp;
			if (ray.dest.dot(faces[0].norm) < 0 && ray.dest.getx())
			{	
				t = (faces[0].pt.getx() - ray.pos.getx()) / ray.dest.getx();
				temp = (ray.pos + ray.dest * t) - faces[0].pt;
				if ( temp.gety() > 0 && temp.getz() > 0 && 
					temp.gety() < faces[0].v && temp.getz() < faces[0].u)
				{
					_face_id = 0;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[1].norm) < 0 && ray.dest.getz())
			{	
				t = (faces[1].pt.getz() - ray.pos.getz()) / ray.dest.getz();
				temp = (ray.pos + ray.dest * t) - faces[1].pt;
				if ( temp.gety() > 0 && temp.getx() > 0 && 
					temp.gety() < faces[1].v && temp.getx() < faces[1].u)
				{
					_face_id = 1;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[2].norm) < 0 && ray.dest.gety())
			{	
				t = (faces[2].pt.gety() - ray.pos.gety()) / ray.dest.gety();
				temp = (ray.pos + ray.dest * t) - faces[2].pt;
				if ( temp.getx() > 0 && temp.getz() > 0 && 
					temp.getx() < faces[2].u && temp.getz() < faces[2].v)
				{
					_face_id = 2;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[3].norm) < 0 && ray.dest.getz())
			{	
				t = (faces[3].pt.getz() - ray.pos.getz()) / ray.dest.getz();
				temp = (ray.pos + ray.dest * t) - faces[3].pt;
				if ( temp.getx() < 0 && temp.gety() < 0 && 
					temp.getx() > faces[3].u && temp.gety() > faces[3].v)
				{
					_face_id = 3;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[4].norm) < 0 && ray.dest.getx())
			{	
				t = (faces[4].pt.getx() - ray.pos.getx()) / ray.dest.getx();
				temp = (ray.pos + ray.dest * t) - faces[4].pt;
				if ( temp.getz() < 0 && temp.gety() < 0 && 
					temp.getz() > faces[4].u && temp.gety() > faces[4].v)
				{
					_face_id = 4;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			if (ray.dest.dot(faces[5].norm) < 0 && ray.dest.gety())
			{	
				t = (faces[4].pt.gety() - ray.pos.gety()) / ray.dest.gety();
				temp = (ray.pos + ray.dest * t) - faces[5].pt;
				if ( temp.getz() < 0 && temp.getx() < 0 && 
					temp.getx() > faces[5].u && temp.getz() > faces[5].v)
				{
					_face_id = 5;
					_t = t;
					_id = _id_save + _face_id;
					return _t;
				}
			}
			return -1;
		}


		inline virtual const types::Vect3df& GetNorm(const types::Vect3df& pt)
		{
			// FIXME.
			_norm = faces[_face_id].norm;
			_bump->perturb(pt, _norm);
			return _norm;
		}

	private:
		unsigned int	_id_save;
		const maintance::Box& _box;
		face_box		faces[6];
		unsigned char	_face_id;
		types::Vect3df	_norm;
	}; 
}

#endif // _RTBOX_HH_
