#ifndef _RAYTRACE_RTCAM_HH
#define _RAYTRACE_RTCAM_HH_

/*---------------------------------------------------------------------*\
|Classe permettant de generer les rayons a lancer en fonction de la     |
|position de la camera et des dimensions du buffer d'affichage          |
|Le rayon utilise est toujours le meme pour eviter des allocations      |
|memoires inutiles.						        |
\*---------------------------------------------------------------------*/

#include <cassert>
#include <cmath>
#include "scene.hh"
#include "camera.hh"
#include "ray.hh"
#include "types.hh"
//#include "mathslib.hh"

namespace raytrace
{
	class RtCam
	{
	public:
		RtCam(const maintance::Camera* cam, 
			unsigned int width, unsigned int height)
		{
			assert(cam);
			_cam = cam;
			_width = width;
			_height = height;
			_ratio = (float)_height/_width;
			update();
		}

	protected:
		const maintance::Camera* _cam;
		Ray			_ray;
		types::Vect3df	_uleft;
		types::Vect3df	_xaxe;
		types::Vect3df	_yaxe;

		unsigned int _width;
		unsigned int _height;
		float	_ratio;

	public:
		inline void SetCam(maintance::Camera* cam)
		{ assert(cam); _cam = cam;}

		void update();


		inline Ray& eval_ray(float x, float y);
	};

	inline Ray& RtCam::eval_ray(float x, float y)
	{ 
		_ray.dest = _uleft;
		_ray.dest += _xaxe * x / (float)_width;
		_ray.dest += _yaxe * y / (float)_height;
		_ray.dest.normalise();
		return _ray;
	}
}
#endif // _RAYTRACE_RTCAM_HH_
