#ifndef _RTCYLINDER_HH_
#define _RTCYLINDER_HH_

//#include <xutility>
//#include <algorithm>
#include "rtobj.hh"
#include "cylinder.hh"
#include "raytracer.hh"
#include "types.hh"
#include "toolance/fsqrt.hh"

#define MIN(a, b) ((a) <= (b) ? (a) : (b))

namespace raytrace
{
	class Raytracer;

	class RtCylinder : public RtObject
	{
	public:
		RtCylinder (maintance::Cylinder& obj)
			: RtObject(obj), _cyl(obj)
		{}

		virtual ~RtCylinder()
		{}

	protected:
		inline void	selectUV(float& u, float& v, const types::Vect3df& vect)
		{
			switch (_cyl.getOrient())
			{
			case maintance::Cylinder::X_AXIS:
				u = vect.gety();
				v = vect.getz();
				break;
			case maintance::Cylinder::Y_AXIS:
				u = vect.getx();
				v = vect.getz();
				break;
			case maintance::Cylinder::Z_AXIS:
				u = vect.getx();
				v = vect.gety();
				break;
			}
		}

		inline float	checkHeight(float t, const Ray& ray) const
		{
			if (_cyl.GetHeight() == 0.0f)
				return t;
			// to be sure we dont compute a unhit cylinder ...
			if (t < 0)
				return -1;
			float	l;
			switch (_cyl.getOrient())
			{
			case maintance::Cylinder::X_AXIS:
				l = (ray.pos - _pos.GetTrans() + ray.dest * t).getx();
				break;
			case maintance::Cylinder::Y_AXIS:
				l = (ray.pos - _pos.GetTrans() + ray.dest * t).gety();
				break;
			case maintance::Cylinder::Z_AXIS:
				l = (ray.pos - _pos.GetTrans() + ray.dest * t).getz();
				break;
			default:
				l = - 1;
				break;
			}
			if (l < 0 || l > _cyl.GetHeight())
				return -1;
			return t;
		}

	public:

		virtual float		primaryHit(Ray& ray)
		{
			float	ud, vd;
			//      static float	up, vp;
			float a, b, det;

			if (_update)
			{
				selectUV(_up, _vp, ray.pos - _pos.GetTrans());
				_c = _up * _up + _vp * _vp - _cyl.GetRadius() * _cyl.GetRadius();
				_update = false;
			}

			selectUV(ud, vd, ray.dest);

			a = ud * ud + vd * vd;
			b = 2 * (_up * ud + _vp * vd);

			det = b * b - 4 * a * _c;
			if (det <= 0)
				return -1;
			else
			{
				float	s_det = math::fsqrt(det);
				float	t;
				t = (-b + MIN(s_det, -s_det)) / (2 * a);
				return (_t = checkHeight(t, ray));
			}
		}


		virtual float		secondaryHit(Ray& ray)
		{
			float	a, b, c, det;
			float	ud, vd, up, vp;
			selectUV(ud, vd, ray.dest);
			selectUV(up, vp, ray.pos - _pos.GetTrans());

			a = ud * ud + vd * vd;
			b = 2 * (up * ud + vp * vd);
			c = up * up + vp * vp - _cyl.GetRadius() * _cyl.GetRadius();

			det = b * b - 4 * a * c;
			if (det <= 0)
				return -1;
			else
			{
				float	s_det = math::fsqrt(det);
				float t = (-b + MIN(s_det, -s_det)) / (2 * a);
				return t;
				//return (_t = checkHeight(t, ray));
			}
		}

		/*
		virtual bool		isHitShad(Ray& ray)
		{
		float	a, b, c, det;
		float	ud, vd, up, vp;
		selectUV(ud, vd, ray.dest);
		selectUV(up, vp, ray.pos - _pos.GetTrans());

		a = ud * ud + vd * vd;
		b = 2 * (up * ud + vp * vd);
		c = up * up + vp * vp - _cyl.GetRadius() * _cyl.GetRadius();

		det = b * b - 4 * a * c;
		if (det <= 0)
		return false;
		else
		{
		float	s_det = sqrt(det);
		float	t;
		t = (-b + std::min(s_det, -s_det)) / (2 * a);
		if (checkHeight(t, ray) < 0)
		return false;
		}
		return true;
		}
		*/

		virtual const types::Vect3df& GetNorm(const types::Vect3df& pt)
		{
			switch (_cyl.getOrient())
			{
			case maintance::Cylinder::X_AXIS:
				_norm.set(0, pt.gety(), pt.getz());
				break;
			case maintance::Cylinder::Y_AXIS:
				_norm.set(pt.getx(), 0, pt.getz());
				break;
			case maintance::Cylinder::Z_AXIS:
				_norm.set(pt.getx(), pt.gety(), 0);
				break;
			}
			_norm.normalise();
			_bump->perturb(pt, _norm);
			return _norm;
		}

	protected:
		float	_up, _vp;
		float	_c;
		types::Vect3df	_norm;
		const maintance::Cylinder& _cyl;
	}; 
}

#endif // _RTCYLINDER_HH_
