#ifndef _RTDISC_HH_
#define _RTDISC_HH_

#include "rtobj.hh"
#include "plane.hh"
#include "disc.hh"
#include "raytracer.hh"

namespace raytrace
{

	class Raytracer;

	class RtDisc : public RtObject
	{

	public:
		RtDisc (maintance::Disc& obj)
			: RtObject(obj), 
			_disc(obj)
		{
			_dv = 0;
			_r2 = _disc.getRadius() * _disc.getRadius();
		}

		virtual ~RtDisc()
		{}

		virtual float		primaryHit(Ray& ray)
		{
			float dv;
			static types::Vect3df vtemp;

			if (_update)
			{
				_norm = _disc.getNormal();
				_update = false;
			}

			// the dot product between ray and normal == 0, ray and plan are parallel
			dv = _norm.dot(ray.dest);
			if (dv == 0)
				return -1;
			types::Vect3df::sub(vtemp, ray.pos, _pos.GetTrans());

			_t = - _norm.dot(vtemp) / dv;

			types::Vect3df::addmult(vtemp, ray.pos, ray.dest, _t);
			types::Vect3df::sub(vtemp, vtemp, _pos.GetTrans());
			//types::Vect3df temp = ray.pos + ray.dest * _t - _pos.GetTrans();

			if (vtemp.dot(vtemp) > _r2)
				_t = -1;
			return _t;
		}


		virtual float		secondaryHit(Ray& ray)
		{
			float dv;
			static types::Vect3df vtemp;

			dv = _norm.dot(ray.dest);
			if (dv == 0)
				return -1;

			types::Vect3df::sub(vtemp, ray.pos, _pos.GetTrans());
			_t = - _norm.dot(vtemp) / dv;

			types::Vect3df::addmult(vtemp, ray.pos, ray.dest, _t);
			types::Vect3df::sub(vtemp, vtemp, _pos.GetTrans());
			//types::Vect3df temp = ray.pos + ray.dest * _t - _pos.GetTrans();

			if (vtemp.dot(vtemp) > _r2)
				_t = -1;

			return _t;
		}

		/*
		virtual bool		isHitShad(Ray& ray)
		{
		static types::Vect3df vtemp;

		float dv;
		float t;
		dv = _norm.dot(ray.dest);
		if (dv == 0)
		return false;

		types::Vect3df::sub(vtemp, ray.pos, _pos.GetTrans());
		t = - _norm.dot(vtemp) / dv;
		//t = -(_norm.dot(ray.pos  - _pos.GetTrans())) / dv;
		if (t < 0)
		return false;
		//types::Vect3df temp = ray.pos + ray.dest * t - _pos.GetTrans();

		types::Vect3df::addmult(vtemp, ray.pos, ray.dest, t);
		types::Vect3df::sub(vtemp, vtemp, _pos.GetTrans());

		if (vtemp.dot(vtemp) > _r2)
		return false;
		return true;
		}*/


		inline virtual const types::Vect3df& GetNorm(const types::Vect3df& pt)
		{
			// FIXME.
			_norm = _disc.getNormal();
			_bump->perturb(pt, _norm);
			return  _norm.normalise();
		}

	private:
		float _dv;
		float _r2;
		const maintance::Disc& _disc;
		types::Vect3df _norm;
	};
}

#endif // _RTDISC_HH_
