// rtobj.hh for raytance
//
// Started on  Fri Aug 23 19:50:51 2002
//  by Laurent CORBES {Caf'} caf@morkitu.com
// Last update Sun Sep 22 22:23:12 2002 alexandre chouippe
//  by Laurent CORBES {Caf'} caf@morkitu.com
// Short Comment: objet de base pour raytracer

#ifndef _RAYTRACE_RTOBGJ_HH_
#define _RAYTRACE_RTOBGJ_HH_

#include "object.hh"
#include "types.hh"
#include "ray.hh"
#include "bumper.hh"
#include "systance/pixel.hh"


namespace raytrace
{

	class Raytracer; // define Raytracer class because of double inclusion

	class RtObject
	{
	public:
		RtObject(maintance::Object& obj) 
			:_pos(obj.GetTransformer().GetPos()), _update(true),
			_shader(&(obj.GetShader()))
		{
			static unsigned int id = 0;

			_id = ++id;

			if (obj.GetType() == maintance::BOX)
				id += 6;
			_t = -1;

			_bump = new Bumper();
			cast_shadow = true;
			receive_shadow = true;

		}

		virtual ~RtObject() {}

		const maintance::Shader*	GetShader() const;
		float		GetT() const;
		void		update();
		unsigned int	GetId() const;


		// Anciennes mthodes
		/*
		virtual bool		isHitShad(Ray& ray) = 0;
		virtual float		hitDist(Ray& ray) = 0;
		virtual float		hitDistRefl(Ray& ray) = 0;
		*/
		// Nouvelles mthodes (correction des ombres)
		virtual float		primaryHit(Ray& ray) = 0;
		virtual float		secondaryHit(Ray& ray) = 0;

		virtual const types::Vect3df& GetNorm(const types::Vect3df& pt) = 0;
		virtual void		SetUV(const types::Vect3df& pt, float& u, float& v);


		bool cast_shadow;
		bool receive_shadow;

		void SetBump( Bumper* bump)
		{
			_bump = bump;
		}

	protected:
		//Ray				_ray;		// current ray
		unsigned int		_id;		// id of the object
		float			_t;		// the result of tests
		const maintance::Position&	_pos;		// current pos
		types::Vect3df		_norm;		// normal to be return
		bool			_update;
		const maintance::Shader*	_shader;	// rtobj shader.
		Bumper*			_bump;


	};

	inline const maintance::Shader*	RtObject::GetShader() const
	{
		return _shader;
	}

	inline float		RtObject::GetT() const
	{
		return _t;
	}

	inline void			RtObject::update()
	{
		_update = true;
	}

	inline unsigned	RtObject::GetId() const
	{
		return _id;
	}

	inline void		RtObject::SetUV(const types::Vect3df& pt, float& u, float& v)
	{
		u = pt.getx() * 0.1f ; v = pt.getz() * 0.1f;

	}

}

#endif //_RAYTRACE_RTOBGJ_HH_
