#ifndef _RTOBJFACTORY_HH_
#define _RTOBJFACORY_HH_

#include "rtobj.hh"
#include "object.hh"
#include "rtsphere.hh"
#include "rtcylinder.hh"
#include "rtplane.hh"
#include "rtbox.hh"
#include "rtdisc.hh"
#include "base.hh"

namespace raytrace
{
	class RtObjFactory
	{
	public:
		RtObject* MakeRtObj(maintance::Object* obj)
		{
			switch (obj->GetType())
			{
			case maintance::SPHERE:
				return MakeRtSphere(static_cast<maintance::Sphere*>(obj));
			case maintance::CYLINDER:
				return MakeRtCylinder(static_cast<maintance::Cylinder*>(obj));
			case maintance::PLANE:
				return MakeRtPlane(static_cast<maintance::Plane*>(obj));
			case maintance::DISC:
				return MakeRtDisc(static_cast<maintance::Disc*>(obj));
			case maintance::BOX:
				return MakeRtBox(static_cast<maintance::Box*>(obj));
			default:
				return 0;
			}
		}


		RtObject* MakeRtSphere(maintance::Sphere* obj)
		{
			return new RtSphere(*obj);
		}

		RtObject* MakeRtCylinder(maintance::Cylinder* obj)
		{
			return new RtCylinder(*obj);
		}

		RtObject* MakeRtPlane(maintance::Plane* obj)
		{
			return new RtPlane(*obj);
		}

		RtObject* MakeRtDisc(maintance::Disc* obj)
		{
			return new RtDisc(*obj);
		}

		RtObject* MakeRtBox(maintance::Box* obj)
		{
			return new RtBox(*obj);
		}


		void AddObj(RtSphere** sphere_list, 
			RtPlane** plane_list, 
			RtBox** box_list,
			RtDisc** disc_list,
			RtCylinder** cylinder_list,
			maintance::Object* obj, 
			unsigned int max_nb_obj)
		{
			unsigned int i = 0;

			switch (obj->GetType())
			{
			case maintance::SPHERE:
				while (sphere_list[i] && i < max_nb_obj)
					i++;
				sphere_list[i] = new RtSphere(*(static_cast<maintance::Sphere*>(obj)));
				break;

			case maintance::CYLINDER:
				while (cylinder_list[i] && i < max_nb_obj)
					i++;
				cylinder_list[i] = new RtCylinder(*(static_cast<maintance::Cylinder*>(obj)));
				break;

			case maintance::PLANE:
				while (plane_list[i] && i < max_nb_obj)
					i++;
				plane_list[i] = new RtPlane(*(static_cast<maintance::Plane*>(obj)));
				break;


			case maintance::DISC:
				while (disc_list[i] && i < max_nb_obj)
					i++;
				disc_list[i] = new RtDisc(*(static_cast<maintance::Disc*>(obj)));
				break;

				break;
				//return MakeRtDisc(static_cast<maintance::Disc*>(obj));
			case maintance::BOX:
				while (box_list[i] && i < max_nb_obj)
					i++;
				box_list[i] = new RtBox(*(static_cast<maintance::Box*>(obj)));
				break;

			default:
				break;
			}

		}


	};

}

#endif // _RTOBJFACTORY_HH_
