#ifndef _RTPLANE_HH_
#define _RTPLANE_HH_

#include "rtobj.hh"
#include "bumperwave.hh"
#include "plane.hh"
#include "raytracer.hh"

namespace raytrace
{

	class Raytracer;

	class RtPlane : public RtObject
	{

	public:
		RtPlane (maintance::Plane& obj)
			: RtObject(obj), 
			_plane(obj)
		{
			_dv = 0;
			// -- FixME !!! --
			_bump = new BumperWave();
			// ------------- 

		}

		virtual ~RtPlane()
		{}

		virtual float		primaryHit(Ray& ray)
		{
			float dv;
			static types::Vect3df vtemp;

			if (_update)
			{
				//_norm = _plane.getNormal() * _plane.GetTransformer().GetRot();
				_norm = _plane.getNormal();
				_norm.normalise();
				_update = false;
			}

			// the dot product between ray and normal == 0, ray and plan are parallel

			dv = _norm.dot(ray.dest);
			if (dv == 0)
				return -1;
			types::Vect3df::sub(vtemp, ray.pos, _plane.GetTransformer().GetPos().GetTrans());

			//_t = -(_norm.dot(ray.pos  - _plane.GetTransformer().GetPos().GetTrans())
			//     ) / dv;
			_t = - _norm.dot(vtemp) / dv;

			return _t;
		}




		virtual float		secondaryHit(Ray& ray)
		{
			float dv;

			dv = _norm.dot(ray.dest);
			if (dv == 0)
				return -1;
			_t = -(_norm.dot(ray.pos  - _plane.GetTransformer().GetPos().GetTrans())
				) / dv;
			return _t;
		}


		inline virtual const types::Vect3df& GetNorm(const types::Vect3df& pt)
		{
			_norm = _plane.getNormal();
			_bump->perturb(pt, _norm);
			return  _norm;
		}

	private:
		float _dv;
		const maintance::Plane& _plane;
		types::Vect3df _norm;
	};
}

#endif // _RTPLANE_HH_
