#ifndef _RTSPHERE_HH_
#define _RTSPHERE_HH_

#include "rtobj.hh"
#include "sphere.hh"
#include "raytracer.hh"
#include "toolance/fsqrt.hh"
//#include "mathslib.hh"

#define MIN(a, b) ((a) <= (b) ? (a) : (b))


namespace raytrace
{

	class RtSphere : public RtObject
	{

	public:
		RtSphere (maintance::Sphere& obj)
			: RtObject(obj), _sph(obj)
		{

		}

		virtual ~RtSphere()
		{}

		virtual float		hitDist(Ray& ray) 
		{
			if (_update)
			{
				types::Vect3df::sub(_oc, _pos.GetTrans(), ray.pos);
				//_oc =  _pos.GetTrans() - ray.pos;
				_loc = _oc.dot(_oc);
				_update = false;
			}
			float tca = _oc.dot(ray.dest);
			if (tca < 0)
				return -1;
			float d = _loc - tca * tca;
			float thc = _sph.GetRadius() * _sph.GetRadius() - d;
			if (thc < 0)
				return -1;
			_t = tca - math::fsqrt(thc);
			return _t;
		}


		virtual float		primaryHit(Ray& ray)
		{
			if (_update)
			{
				types::Vect3df::sub(_oc, _pos.GetTrans(), ray.pos);
				//_oc =  _pos.GetTrans() - ray.pos;
				_loc = _oc.dot(_oc);
				_update = false;
			}
			float tca = _oc.dot(ray.dest);
			if (tca < 0)
				return -1;
			float d = _loc - tca * tca;
			float thc = _sph.GetRadius() * _sph.GetRadius() - d;
			if (thc < 0)
				return -1;
			_t = tca - math::fsqrt(thc);
			return _t;
		}

		virtual float		secondaryHit(Ray& ray)
		{
			static types::Vect3df oc;
			types::Vect3df::sub(oc, _pos.GetTrans(), ray.pos);
			//oc =  _pos.GetTrans() - ray.pos;
			float loc = oc.dot(oc);
			float tca = oc.dot(ray.dest);
			if (tca < 0)
				return -1;
			float d = loc - tca * tca;
			float thc = _sph.GetRadius() * _sph.GetRadius() - d;
			if (thc < 0)
				return -1;
			_t = tca - math::fsqrt(thc);
			return _t;
		}

		/*
		virtual float		hitDistRefl(Ray& ray) 
		{
		static types::Vect3df oc;
		types::Vect3df::sub(oc, _pos.GetTrans(), ray.pos);
		//oc =  _pos.GetTrans() - ray.pos;
		float loc = oc.dot(oc);
		float tca = oc.dot(ray.dest);
		if (tca < 0)
		return -1;
		float d = loc - tca * tca;
		float thc = _sph.GetRadius() * _sph.GetRadius() - d;
		if (thc < 0)
		return -1;
		_t = tca - sqrt(thc);
		return _t;
		}*/



		/*
		virtual bool		isHitShad(Ray& ray) 
		{ 
		static types::Vect3df oc;
		float tmp;
		types::Vect3df::sub(oc, _pos.GetTrans(), ray.pos);
		//oc =  _pos.GetTrans() - ray.pos;
		float loc = oc.dot(oc);
		float tca = oc.dot(ray.dest);
		if (tca < 0)
		return false;
		float d = loc - tca * tca;
		float thc = _sph.GetRadius() * _sph.GetRadius() - d;
		if (thc < 0)
		return false;
		return true;
		} */


		inline virtual const types::Vect3df& GetNorm(const types::Vect3df& pt)
		{
			types::Vect3df::sub(_norm, pt, _pos.GetTrans());

			_norm /= _sph.GetRadius();
			_bump->perturb(pt, _norm);

			//  _norm = (pt - _pos.GetTrans()).normalise();
			return _norm;
		}

	private:
		float	_loc;
		types::Vect3df _norm;
		types::Vect3df _oc;
		types::Vect3df _vdelta;
		const maintance::Sphere& _sph;
	};

}

#endif // _RTSPHERE_HH_
