#include <conio.h>
#include "systance/window.hh"
#include "systance/time.hh"
#include "systance/frame_buffer.hh"
#include "raytance-scene.hh"
#include "maintance/base/scene.hh"
#include "maintance/sequencer.hh"
#include "maintance/sequencer-scene.hh"
#include "toolance/fsqrt.hh"
#include "minifmod/minifmod.h"
#include "maintance/font.hh"


#define MINIRAYTANCE
#define RAY_DEBUG
#define USEFMOD
#define USEMEMLOAD


int		verbose = 0;

#define WIDTH	640
#define HEIGHT	480
#define BPP	16
#define PIXEL	1
//#define FULLSCREEN	false
#define FULLSCREEN	true


// ---------- FMOD truc
#ifndef USEMEMLOAD

unsigned int fileopen(char *name)
{
	return (unsigned int)fopen(name, "rb");
}

void fileclose(unsigned int handle)
{
	fclose((FILE *)handle);
}

int fileread(void *buffer, int size, unsigned int handle)
{
	return fread(buffer, 1, size, (FILE *)handle);
}

void fileseek(unsigned int handle, int pos, signed char mode)
{
	fseek((FILE *)handle, pos, mode);
}

int filetell(unsigned int handle)
{
	return ftell((FILE *)handle);
}
#else
typedef struct 
{
	int length;
	int pos;
	void *data;
} MEMFILE;


unsigned int memopen(char *name)
{
	MEMFILE *memfile;

	memfile = (MEMFILE *)calloc(sizeof(MEMFILE),1);

	{	// hey look some load from resource code!
		HRSRC		rec;
		HGLOBAL		handle;

		rec = FindResource(NULL, MAKEINTRESOURCE(101), RT_RCDATA);
		handle = LoadResource(NULL, rec);
		
		memfile->data = LockResource(handle);
		memfile->length = SizeofResource(NULL, rec);
		memfile->pos = 0;
	}
	return (unsigned int)memfile;
}
void memclose(unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;
	free(memfile);
}

int memread(void *buffer, int size, unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	if (memfile->pos + size >= memfile->length)
		size = memfile->length - memfile->pos;

	memcpy(buffer, (char *)memfile->data+memfile->pos, size);
	memfile->pos += size;
	
	return size;
}

void memseek(unsigned int handle, int pos, signed char mode)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	if (mode == SEEK_SET) 
		memfile->pos = pos;
	else if (mode == SEEK_CUR) 
		memfile->pos += pos;
	else if (mode == SEEK_END)
		memfile->pos = memfile->length + pos;

	if (memfile->pos > memfile->length)
		memfile->pos = memfile->length;
}

int memtell(unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	return memfile->pos;
}
#endif


// FMOD truc

int main (int argc, char **argv)
{
	// Precalc pour sqrt
	math::initFsqrtTab();
	// loading xml file
	maintance::Scene* scene = load_1();
	maintance::Scene* scene2 = load_2();
	maintance::Scene* scene3 = load_3();
	maintance::Scene* scene4 = load_4();

	if (!scene)
		exit (1);
	scene->SetTime(0.0f);
	scene->print(std::cout);
	scene2->print(std::cout);
	scene3->print(std::cout);
	scene4->print(std::cout);

	// Core system
	systance::Time	time;
	systance::UnixWindow	win;
	systance::FrameBuffer	fb;

	if ((!win.Create(WIDTH, HEIGHT, BPP, FULLSCREEN)) ||
		(!(fb.Create(false, PIXEL, WIDTH, HEIGHT))))
		exit (2);

	BuildFont(win.hDC, "Verdana");
	types::Vect3df disturb(20.f, 20.f, 0.f);
	types::Vect3df pos(200.f, 400.f, 0.f);
	std::string me("");

	//messages
	std::string ce("Cymagine & Epidemic");
	std::string pr("Presents");
	std::string sp("Synthesis Party 4");
	std::string da("february 20,21,22 2004");
	std::string pa("45Km from Paris");
	std::string url("http://wwww.synthesis-party.net/");
	std::string in("SP4-03:High ra(y)tance intro");
	std::string ct("Come TO or DIE");
	std::string pi("PiPiDemic RP");

	// fb.SetBase(base);

// initialisation des sequences
	maintance::Sequencer	rootseq;
	maintance::SceneSequencer sscene(*scene, &fb);
	rootseq.AddChild(sscene);
	types::Curve<float> scurve;
	scurve.add(0, 0);
	scurve.add(300, 100);
	scurve.finish();
	sscene.SetCurve(scurve);
	
	maintance::SceneSequencer sscene2(*scene2, &fb);
	types::Curve<float>	scurve2;
	scurve2.add(0, 0);
	scurve2.add(400, 100);
	scurve2.finish();
	sscene2.SetCurve(scurve2);

	maintance::SceneSequencer sscene3(*scene3, &fb);
	types::Curve<float>	scurve3;
	scurve3.add(0, 0);
	scurve3.add(160, 100);
	scurve3.finish();
	sscene3.SetCurve(scurve3);

	maintance::SceneSequencer sscene4(*scene4, &fb);
	types::Curve<float>	scurve4;
	scurve4.add(0, 0);
	scurve4.add(390, 100);
	scurve4.finish();
	sscene4.SetCurve(scurve4);

// ----------- SCENE LOADING

// ----------- FMOD
#ifdef USEFMOD
	FMUSIC_MODULE *mod;

#ifndef USEMEMLOAD
	FSOUND_File_SetCallbacks(fileopen, fileclose, fileread, fileseek, filetell);
#else
	FSOUND_File_SetCallbacks(memopen, memclose, memread, memseek, memtell);
#endif
#endif

	mod = FMUSIC_LoadSong("IDR_RCDATA1", NULL); //sampleloadcallback);
	if (!mod)
	{
		std::cout << "Error loading song" << std::endl;
	}
#ifdef USEFMOD
	FMUSIC_PlaySong(mod);
#endif

// ----------- MAIN LOADING
	// compteur de frames
	unsigned int	frame_draw = 0;
	// le temps de la demo avant de boucler
	float maxtime = 60.f;
	float	time_last = -1.5f;
	float	curtime = 0.0f;
#ifdef USEFMOD
	int		ord = 0, row = 0;
#endif

	time.Init();
	while(!win.IsQuit())
	{

#ifdef USEFMOD
		ord = FMUSIC_GetOrder(mod);
		row = FMUSIC_GetRow(mod);
		curtime = (float)FMUSIC_GetTime(mod) / 1000.0f;
#else
		time.Lock();
		curtime = time.Get();
#endif

#ifdef RAY_DEBUG
		if (curtime - time_last >= 1.0f)
		{
			std::cout << frame_draw << " fps, " <<  fb.GetRayCount() << " rayz, " << curtime << " sec" << std::endl;
			frame_draw = 0;
			time_last = curtime;
		}
		else
			frame_draw++;
//		std::cout << curtime << std::endl;
#endif RAY_DEBUG

// intro total duration 0.0f -> 53.0f

		if ((curtime >= 0.f) && (curtime <= 2.5f))
		{
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glClearColor(0.f, 0.f, 0.f, 1.f);
			glColor3f(0.f, 0.f, 0.f);
			glBegin(GL_QUADS);
			glVertex3f(0.f, 0.f, -5.f);
			glVertex3f(0.f, 3.f, -5.f);
			glVertex3f(3.f, 3.f, -5.f);
			glVertex3f(3.f, 0.f, -5.f);
			glEnd();
		}
		if ((curtime >= 2.5f ) && (curtime <= 14.5f))
		{
			rootseq.Draw((curtime - 2.5f) * 24.0f);
			 if ((curtime >= 5.f) && (curtime <= 10.0f))
         me = ce;
			 else
				 if ((curtime >= 10.0f) && (curtime <= 14.5f))
					 me = pr;
				 else
					 me = "";
		}
		else
			if ((curtime >= 14.5f ) && (curtime <= 29.5f))
			{
				sscene2.Draw((curtime - 14.5f) * 24.0f);
				if ((curtime >= 14.5f) && (curtime <= 22.f))
					me = sp;
				else
					if ((curtime >= 25.f) && (curtime <= 28.f))
						me = pi;
					else
						me = "";
			}
		else
			if ((curtime >= 29.5f ) && (curtime <= 36.5f))
			{
				sscene3.Draw((curtime - 29.5f) * 24.0f);
				if ((curtime >= 29.5f) && (curtime <= 33.f))
						me = da;
					else
					if ((curtime >= 33.f) && (curtime <= 36.5f))
						me = pa;
					else
							me = "";
			}
		else
			if ((curtime >= 36.5f ))
			{
				sscene4.Draw((curtime - 36.5f) * 24.0f);
//				if ((curtime >= 36.5f) && (curtime <= 39.f))
//					me = pa;
//				else
					if ((curtime >= 39.f) && (curtime <= 43.f))
						me = in;
					else
						if ((curtime >= 45.f) && (curtime <= 45.3f))
							me = ct;
					else
						if ((curtime >= 46.5f) && (curtime <= 46.8f))
							me = ct;
						else
							if ((curtime >= 48.f))
							{
								pos = types::Vect3df(100, 400,0);
								me = url;
							}
							else
								me = "";
			}
	
			
//			sscene4.Draw(curtime * 24.f);
		
		curverand(curtime, me, disturb, pos);

// affichage de framebuffer
		win.Draw();

//end of intro
		if (curtime >= maxtime)
			break;

#ifndef USEFMOD
		time.UnLock();
#endif
	}


#ifdef USEFMOD
	FMUSIC_FreeSong(mod);
#endif
	win.Destroy();
	delete scene;
	exit (0);
}
