//	frame_buffer.cc
//
//	Henry PRECHEUR - E-mail: henry.precheur@epita.fr
//	last modification Time-stamp: <09-mai-2003 21:51:45 alex>

#include "frame_buffer.hh"
#include <cmath>
//#include <cassert>
#include <iostream>
#include <GL/gl.h>
#include "maintance/raytrace/raytracer.hh"


// #define ENABLE_TEXT
#define MAX_DIFF	2

namespace systance
{
	FrameBuffer::FrameBuffer()
	{
		_Ray = 0;
		_MaxRecursion = 0;
		_date = 0;

		// Temporaire Initialisation de la texture
		int i, j, c;

		for (i = 0; i < 64; i++)
			for (j = 0; j < 64; j++)
			{
				c = ((((i&0x8)==0)^((j&0x8))==0)) * 255;
				/*	  if (c)
				c -= ((cos(j * 2) + 1) * (sin(i * 2) + 1) / 2.0) * 100.0;
				else
				c = ((cos(i) + 1) * (sin(j) + 1) / 2.0) * 100;*/
				texture[i][j][0] = c;
				texture[i][j][1] = c;
				texture[i][j][2] = c;
				texture[i][j][3] = 255;
			}

	}

	FrameBuffer::~FrameBuffer()
	{
		delete []_QuadTree;
		delete []_PixelBuffer;
	}

	bool	FrameBuffer::Create(bool wire,
		unsigned int pixel_ratio,
		unsigned int width,
		unsigned int height)
	{
//		assert(_MaxRecursion == 0);
		width /= pixel_ratio;
		height /= pixel_ratio;

		_PixelSize = pixel_ratio;
		_Wire = wire;
		unsigned int	max = (width > height ? width : height);
		unsigned int	max_recursion;
		for (max_recursion = 0;
			((unsigned int)1 << max_recursion) < max; max_recursion++);

			_MaxRecursion = max_recursion;
		_Base = max_recursion / 3 + 1;
		_WidthAspect = width;
		_HeightAspect = height;

		_QuadCount = 1;
		unsigned int	r = 1;
		for (unsigned int i = 0; i < max_recursion; i++)
		{
			r *= 4;
			_QuadCount += r;
		}

		unsigned int square_r = (unsigned int)sqrt((float)r);
		_PixelEdgeCount = square_r + 1;
		_PixelCount = _PixelEdgeCount * _PixelEdgeCount;

		// FIXME: catch exception. => or do a new/delete operator to avoid
		// exceptions
		_QuadTree = new _Quad[_QuadCount];
		_PixelBuffer = new Pixel[_PixelCount];

		QuadInit(0, 0, (_PixelEdgeCount - 1) * _PixelEdgeCount,
			_PixelEdgeCount * _PixelEdgeCount - 1, _PixelEdgeCount - 1);
		for (unsigned int i = 0; i < _PixelEdgeCount; i++)
			for (unsigned int j = 0; j < _PixelEdgeCount; j++)
			{
				_PixelBuffer[i * _PixelEdgeCount + j]._PosX = j;
				_PixelBuffer[i * _PixelEdgeCount + j]._PosY = i;
				_PixelBuffer[i * _PixelEdgeCount + j].Date = 0;
			}

			std::cout << __FILE__ << " @ " << __LINE__ << " : FrameBuffer stats"
				<< std::endl;
			std::cout << "QuadCount: " << _QuadCount << std::endl << "PixelCount: "
				<< _PixelCount << std::endl << "PixelEdgeCount: "
				<< _PixelEdgeCount << std::endl;
			std::cout << "Aspect: " << _WidthAspect << " * " << _HeightAspect 
				<< std::endl;
			return true;
	}

	//   0     3
	//   +<----+
	//   |     ^
	//   V     |
	//   +---->+
	//   1     2
	void	FrameBuffer::QuadInit(unsigned int index,
		unsigned int a, unsigned int b,
		unsigned int c, unsigned int d)
	{
		_QuadTree[index]._[0] = _PixelBuffer + a;
		_QuadTree[index]._[1] = _PixelBuffer + b;
		_QuadTree[index]._[2] = _PixelBuffer + c;
		_QuadTree[index]._[3] = _PixelBuffer + d;

		if (((a % _PixelEdgeCount) >= _WidthAspect)
			|| ((a / _PixelEdgeCount) >= _HeightAspect))
			_QuadTree[index].Active = false;
		else
		{
			_QuadTree[index].Active = true;
			//     GROS GORET !!!!
#define MEAN(i, j)	(((i % _PixelEdgeCount + j % _PixelEdgeCount) / 2) +\
	((i / _PixelEdgeCount + j / _PixelEdgeCount) / 2) *\
	_PixelEdgeCount)

			if ((4 * index + 1) < _QuadCount)
			{
				QuadInit(4 * index + 1, a, MEAN(a, b), MEAN(a, c), MEAN(a, d));
				QuadInit(4 * index + 2, MEAN(a, b), b, MEAN(b, c), MEAN(a, c));
				QuadInit(4 * index + 3, MEAN(a, c), MEAN(b, c), c, MEAN(d, c));
				QuadInit(4 * index + 4, MEAN(a, d), MEAN(a, c), MEAN(d, c), d);
			}
#undef MEAN
		}
	}

	void	FrameBuffer::QuadPreTrace(unsigned int index, unsigned int depth)
	{
		if (((4 * index + 1) < _QuadCount) && _QuadTree[index].Active)
		{
			_QuadTree[index].Date = _date;
			if (depth > _Base)
			{
				QuadTrace(4 * index + 1);
				QuadTrace(4 * index + 2);
				QuadTrace(4 * index + 3);
				QuadTrace(4 * index + 4);
			}
			else
			{
				QuadPreTrace(4 * index + 1, depth + 1);
				QuadPreTrace(4 * index + 2, depth + 1);
				QuadPreTrace(4 * index + 3, depth + 1);
				QuadPreTrace(4 * index + 4, depth + 1);
			}
		}
	}

	void	FrameBuffer::Trace()
	{
		_RayCount = 0;
		if (++_date == 0)
			_date = 1;
		//    QuadPreTrace(0, 0);
		Draw();

	}

	void	FrameBuffer::QuadTrace(unsigned int index)
	{
		bool draw = true;
		if (_QuadTree[index].Active)
		{
			for (unsigned int i = 0; i < 4; i++)
			{
				if (_QuadTree[index]._[i]->Date != _date)
				{
					_Ray->RenderPixel(_QuadTree[index]._[i]);
					_QuadTree[index]._[i]->Date = _date;
					_RayCount++;
				}
			}
			_QuadTree[index].Date = _date;
			if ((4 * index + 1) < _QuadCount)
			{
				for (unsigned int i = 0; i < 3 && draw; i++)
					for (unsigned int j = i + 1; j < 4; j++)
						if (_QuadTree[index]._[i]->ObjId !=
							_QuadTree[index]._[j]->ObjId ||
							_QuadTree[index]._[i]->ShadowId !=
							_QuadTree[index]._[j]->ShadowId ||
							_QuadTree[index]._[i]->ReflexId !=
							_QuadTree[index]._[j]->ReflexId || 
							maintance::Color::compare(_QuadTree[index]._[j]->_Color,_QuadTree[index]._[i]->_Color) > MAX_DIFF)
						{
							QuadTrace(4 * index + 1);
							QuadTrace(4 * index + 2);
							QuadTrace(4 * index + 3);
							QuadTrace(4 * index + 4);
							draw = false;
							break;
						}
			}
			if (draw)
				QuadDraw(index);
		}
	}


	void	FrameBuffer::QuadDraw(unsigned int index) const
	{

		if (! _QuadTree[index].Active)
			return;
		/*
		if (((4 * index + 1) < _QuadCount) && 
		_QuadTree[4 * index + 1].Date == _date)
		{
		QuadDraw(4 * index + 1);
		QuadDraw(4 * index + 2);
		QuadDraw(4 * index + 3);
		QuadDraw(4 * index + 4);
		}
		else*/
		{
#ifdef ENABLE_TEXT	
			if (_QuadTree[index]._[0]->ObjId == 0 || _QuadTree[index]._[0]->ObjId == 0xffff - 1)
			{
				glEnd();
				glDisable(GL_TEXTURE_2D);
				glBegin(GL_QUADS);
			}
#endif	
			glColor3fv(_QuadTree[index]._[0]->_Color.GetColor());
			glTexCoord2f(_QuadTree[index]._[0]->_UV._[0], _QuadTree[index]._[0]->_UV._[1]);
			glVertex2f(_QuadTree[index]._[0]->_PosX,
				_QuadTree[index]._[0]->_PosY);

			glColor3fv(_QuadTree[index]._[1]->_Color.GetColor());
			glTexCoord2f(_QuadTree[index]._[1]->_UV._[0], _QuadTree[index]._[1]->_UV._[1]);
			glVertex2f(_QuadTree[index]._[1]->_PosX,
				_QuadTree[index]._[1]->_PosY);

			glColor3fv(_QuadTree[index]._[2]->_Color.GetColor());
			glTexCoord2f(_QuadTree[index]._[2]->_UV._[0], _QuadTree[index]._[2]->_UV._[1]);
			glVertex2f(_QuadTree[index]._[2]->_PosX,
				_QuadTree[index]._[2]->_PosY);

			glColor3fv(_QuadTree[index]._[3]->_Color.GetColor());
			glTexCoord2f(_QuadTree[index]._[3]->_UV._[0], _QuadTree[index]._[3]->_UV._[1]);
			glVertex2f(_QuadTree[index]._[3]->_PosX,
				_QuadTree[index]._[3]->_PosY);

#ifdef ENABLE_TEXT


			if (_QuadTree[index]._[0]->ObjId == 0 || _QuadTree[index]._[0]->ObjId == 0xffff - 1)
			{
				glEnd();
				glEnable(GL_TEXTURE_2D);
				glBegin(GL_QUADS);
			}
#endif
		}
	}

	void	FrameBuffer::Draw()
	{
		static bool init = true;
		if (_Wire)
		{
			glClear(GL_COLOR_BUFFER_BIT);
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		}

		if (init)
		{
			init = false;
			glGenTextures(1, &tex_id);
			glBindTexture(GL_TEXTURE_2D, tex_id);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 64, 64, 0, GL_RGBA, GL_UNSIGNED_BYTE, texture);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		}

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0, _WidthAspect, _HeightAspect, 0, -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
#ifdef ENABLE_TEXT
		glEnable(GL_TEXTURE_2D);
#endif
		glBegin(GL_QUADS);
		glBindTexture(GL_TEXTURE_2D, tex_id);
		QuadPreTrace(0, 0);
		//    QuadDraw(0);
		glEnd();
#ifdef ENABLE_TEXT
		glEnable(GL_TEXTURE_2D);
#endif
	}
}
