//	frame_buffer.hh
//
//	Henry PRECHEUR - E-mail: henry.precheur@epita.fr
//	last modification Time-stamp: <23-Sep-2002 05:38 by preche_h>

#ifndef	_FRAME_BUFFER_HH_
#define	_FRAME_BUFFER_HH_

#include <cassert>
#include <windows.h>
#include <GL/gl.h>
#include "pixel.hh"

// Comment: 
// Frame buffer pour raytance
// il ce lit de haut en bas et de gauche a droite
// 0.0
//    \ ___>
//     |
//     |
//    \/
// sans doute une meilleure version du framebuffer est a faire ?
// du moins henry va falloir expliquer certains trucs

namespace raytrace
{
	class Raytracer;
}

namespace systance
{
	class FrameBuffer
	{
	protected:

		struct _Quad
		{
			Pixel*		_[4];
			unsigned short	Date;
			bool		Active;
		};

	public:
		void	Draw();
		// @ utiliser avec le raytraceur (alex font qu'on voit 2/3
		// details ensemble afin de faire ca bien).
		void	Trace();

	protected:
		// le buffer
		Pixel*		_PixelBuffer;
		_Quad*		_QuadTree;
		raytrace::Raytracer*	_Ray;

		/*
		/!\ Temporaires, tests pour les textures.
		------------------------------------------------------
		*/

		unsigned int tex_id;
		char texture[64][64][4];

		//------------------------------------------------------

		// Profondeur du quadtree.
		unsigned int	_MaxRecursion;
		// FIXME: non utilise pour l'instant.
		unsigned int	_WidthAspect;
		unsigned int	_HeightAspect;

		unsigned int	_PixelEdgeCount;
		unsigned int	_PixelCount;
		unsigned int	_QuadCount;
		unsigned int	_PixelSize;
		unsigned int	_Base;
		unsigned		_RayCount;
		unsigned short	_date;
		bool		_Wire;

		void	QuadInit(unsigned int index,
			unsigned int a, unsigned int b,
			unsigned int c, unsigned int d);
		void	QuadPreTrace(unsigned int index,
			unsigned int depth);
		void	QuadTrace(unsigned int index);
		void	QuadDraw(unsigned int index) const;

	public:
		FrameBuffer();
		~FrameBuffer();

		bool	Create(bool wire,
			unsigned int pixel_ration,
			unsigned int width,
			unsigned int height);
		Pixel&	getPixel(unsigned int x, unsigned int y);
		inline unsigned int	GetWidth() const { return _WidthAspect + 1; }
		inline unsigned int	GetHeight() const { return _HeightAspect + 1; }
		void	Attach(raytrace::Raytracer& ray)
		{
			_Ray = &ray;
		}

		void	SetBase(unsigned int base)
		{
			_Base = base;
		}

		unsigned	GetRayCount() const;
	};

	inline Pixel& FrameBuffer::getPixel(unsigned int x, unsigned int y)
	{
		assert (x < _WidthAspect + 1);
		assert (y < _HeightAspect + 1);
		return _PixelBuffer[x * _PixelEdgeCount + y];
	}

	inline unsigned	FrameBuffer::GetRayCount() const
	{
		return _RayCount;
	}
}

#endif // _FRAME_BUFFER_HH_
