//	time.hh
//
//	Henry PRECHEUR - E-mail: henry.precheur@epita.fr
//	last modification Time-stamp: <08-May-2003 19:52:59 caf>

#ifndef	_TIME_HH_
#define	_TIME_HH_

#ifdef UNIX
#	include <sys/time.h>
#else
#	include <windows.h>
#endif
#include <ctime>
#include <cassert>
#include <iostream>

namespace systance
{
  class Time
  {
  private:
    float	_Time;
    double	_TimeStart;
#ifdef LINUX
    struct timeval	_Tv;
#else
		double	_frequency;
#endif

    bool	_Locked;
    void	_UpdateTime();

  public:
    Time();
    void	Lock();
    void	UnLock();
    void	Init();
    // // time is given in seconds since start of raytance !

    float 	Get();
    void	Sleep(float secs);
  };

  inline   void	Time::_UpdateTime()
  {
    assert(!_Locked);
#ifdef LINUX
    if (gettimeofday(&_Tv, 0) != 0)
      std::cerr << "Error: unable to handle timer!" << std::endl;
    _Time = float((_Tv.tv_sec + (_Tv.tv_usec / 1000000.0f)) - _TimeStart);
#else
		LARGE_INTEGER ticks;

		QueryPerformanceCounter(&ticks);
		_Time = (ticks.QuadPart / _frequency) - _TimeStart;
#endif

  }
  
  inline void	Time::Lock()
  {
    assert(!_Locked);
    _UpdateTime();
    _Locked = true;
  }

  inline void	Time::UnLock()
  {
    assert(_Locked);
    _Locked = false;
  }

  inline float	Time::Get()
  {
    if (!_Locked)
      _UpdateTime();
    return _Time;
  }
}

#endif // _TIME_HH_
