#ifndef _TYPES_CONVERT_HH_
#define _TYPES_CONVERT_HH_
 
#include <cmath>
#include "types.hh"

namespace types
{

  template<typename T>
  Matrix<T> quatToMatrix (const Quaternion<T>& rhs);
  
  template<typename T>
  Quaternion<T> eulerToQuat (const Euler<T>& rhs);

  template<typename T>
  Matrix<T> mulVect3d (const Vect3d<T>& v1, 
		       const Vect3d<T>& v2);

  template<typename T>
  Vect3d<T> mulMatVect(const Matrix<T>& mat,
		       const Vect3d<T>& vect);

#define quatToMatrixf quatToMatrix<float>
#define quatToMatrixd quatToMatrix<double>

#define eulerToQuatf eulerToQuat<float>
#define eulerToQuatd eulerToQuat<double>

#define mulVect3df mulvect3d<float>
#define mulVect3dd mulvect3d<double>

#define mulMatVectf mulMatVect<float>
#define mulMatVectd mulMatVect<double>

  
  template<typename T>
  Matrix<T> quatToMatrix (const Quaternion<T>& rhs)
  {
    Matrix<T> tmp (3, 3);
    tmp(1, 1) = 1 - 2 * rhs.gety() * rhs.gety()
      - 2 * rhs.getz() * rhs.getz();
    tmp(1, 2) = 2 * rhs.getx() * rhs.gety() - 2 * rhs.getz() * rhs.getw();
    tmp(1, 3) = 2 * rhs.getx() * rhs.getz() + 2 * rhs.gety() * rhs.getw();
    tmp(2, 1) = 2 * rhs.getx() * rhs.gety() + 2 * rhs.getz() * rhs.getw();
    tmp(2, 2) = 1 - 2 * rhs.getx() * rhs.getx() 
      - 2 * rhs.getz() * rhs.getz();
    tmp(2, 3) = 2 * rhs.gety() * rhs.getz() - 2 * rhs.getx() * rhs.getw();
    tmp(3, 1) = 2 * rhs.getx() * rhs.getz() - 2 * rhs.gety() * rhs.getw();
    tmp(3, 2) = 2 * rhs.gety() * rhs.getz() + 2 * rhs.getx() * rhs.getw();
    tmp(3, 3) = 1 - 2 * rhs.getx() * rhs.getx() 
      - 2 * rhs.gety() * rhs.gety();
    return tmp;
  }
  
  template<typename T>
  Quaternion<T> eulerToQuat (const Euler<T>& rhs)
  {
    Quaternion<T>    q1(cos(rhs.getv().getx() / 2), sin(rhs.getv().getx() / 2),
			0, 0);
    Quaternion<T>    q2(cos(rhs.getv().gety() / 2), 0, 
			sin(rhs.getv().gety() / 2), 0);
    Quaternion<T>    q3(cos(rhs.getv().getz() / 2), 0,
			0, sin(rhs.getv().getz() / 2));
    Quaternion<T>    ret = q1 * q2 * q3;
    return ret;
  }

  template<typename T>
  Matrix<T> mulVect3d (const Vect3d<T>& v1, 
		       const Vect3d<T>& v2)
  {
    Matrix<T> tmp (3, 3);
    tmp(1, 1) = v1.getx() * v2.getx();
    tmp(1, 2) = v1.getx() * v2.gety();
    tmp(1, 3) = v1.getx() * v2.getz();
    tmp(2, 1) = v1.gety() * v2.getx();
    tmp(2, 2) = v1.gety() * v2.gety();
    tmp(2, 3) = v1.gety() * v2.getz();
    tmp(3, 1) = v1.getz() * v2.getx();
    tmp(3, 2) = v1.getz() * v2.gety();
    tmp(3, 3) = v1.getz() * v2.getz();
    return tmp;
  }
  
  template<typename T>
  Vect3d<T> mulMatVect(const Matrix<T>& mat,
		       const Vect3d<T>& vect)
  {
    Vect3d<T> res;
    assert(mat.getm() == 3 && mat.getn() == 3);
    
    res.setx(mat(1, 1) * vect.getx() + 
	     mat(1, 2) * vect.gety() + 
	     mat(1, 3) * vect.getz());
    res.sety(mat(2, 1) * vect.getx() + 
	     mat(2, 2) * vect.gety() + 
	     mat(2, 3) * vect.getz()); 
    res.setz(mat(3, 1) * vect.getx() + 
	     mat(3, 2) * vect.gety() + 
	     mat(3, 3) * vect.getz());
    return res;
  }
}

#endif // _TYPES_CONVERT_HH_
