//
// curve.hh
// 
// Made by (Alex)
// Login   <alex@Arakis>
// 
// Started on  Mon May  5 21:55:36 2003 (Alex)
//

#ifndef   _MAINTANCE_CURVE_HH_
# define  _MAINTANCE_CURVE_HH_

//#include "rtvector.hh"
//#include <cassert>
#include <vector>

namespace types
{

	// Classe permettant de calculer une courbe. Utilise la mthode de
	// Catmull-Rom pour interpoler les diffrents points
	template <typename T>
	class Curve
	{
	public :    

		// Point clef de l'animation
		typedef struct key
		{
			T elt;
			float frame;
		} AnimKey;


		//---------------------------------------
		Curve()
		{
			AnimKey tmp;
			_max_time = 0;
			_last_h = 0;
			_last_index = 0;
			tmp.frame = 0;
			_finished = false;
			// On ajoute une premire clef qui sera utilise pour le calcul
			// du premier segment par Catmull-Rom
			_curve.push_back(tmp);
		}

		//--------- Retourne la valeur de la courbe au point h --------------
		T eval(float h)
		{
//			assert(_curve.size() > 1 && h >= 0 && _finished);

			if (_curve.size() == 2)
				return _curve[1].elt;
			if (h <= 0)
				return _curve[1].elt;
			if (h >= _max_time)
				return _curve[_curve.size() - 1].elt;

			// Dtermination du t de la courbe
			int key_index = get_index(h);

//			std::cout << " index " << key_index << std::endl;

			float _h = (h - _curve[key_index].frame) / (_curve[key_index + 1].frame 
				- _curve[key_index].frame);
			// Calcul de f(t)
			return _eval(key_index, _h);
		}

		//-------- Mthode  appeler pour prparer la courbe a Catmull-Rom. Il
		//n'est plus possible d'ajouter de clef ensuite ----------------
		void finish()
		{
			_finished = true;
			if (_finished)
				return;
			if (_curve.size() > 2)
			{
				AnimKey tmp;

				_curve[0].elt = _curve[1].elt + (_curve[1].elt - _curve[2].elt);
				_curve[0].frame = _curve[1].frame + (_curve[1].frame - _curve[2].frame);
				tmp.elt = _curve[_curve.size()-1].elt + (_curve[_curve.size()-1].elt - _curve[_curve.size()-2].elt);
				tmp.frame = _curve[_curve.size()-1].frame + (_curve[_curve.size()-1].frame - _curve[_curve.size()-2].frame);
				_curve.push_back(tmp);
			}
		}

		//--------------------------------------------------------------
		void add(float frame, const T& elt)
		{
			AnimKey tmp;
			tmp.elt = elt;
			tmp.frame = frame;
//			assert(frame == 0 || frame > _max_time && !_finished);
			_curve.push_back(tmp);
			_max_time = frame;
		}

	private :
		//---------- Retourne l'index du premier point de la courbe -----------
		int get_index(float h)
		{
			if (_last_h > h)
				_last_index = 0;

			while ((h > _curve[_last_index].frame) && (_last_index < _curve.size() -1))
				_last_index++;
			_last_h = h;
			return --_last_index;
		}

		//--------------------------------------------------------------
		T _eval(int i, float h)
		{
			T res;

			// Interpolation linaire
			//res = _curve[i+1].elt * h + _curve[i].elt * (1 - h);

			/*
			Formule de Catmull-Rom : 
			q(t) = 0.5 * ((-p1 + 3*p2 -3*p3 + p4)*t*t*t
			+ (2*p1 -5*p2 + 4*p3 - p4)*t*t
			+ (-p1+p3)*t
			+ 2*p2)
			*/

			if ((i + 2) > (_curve.size() - 1))
				res = ( (_curve[i].elt * 3 - _curve[i+1].elt * 3 + _curve[i+1].elt - _curve[i-1].elt) * h * h * h
				+ (_curve[i-1].elt * 2 + _curve[i+1].elt * 4 - _curve[i+1].elt - _curve[i].elt * 5) * h * h
				+ (_curve[i+1].elt - _curve[i-1].elt) * h
				+ _curve[i].elt * 2) * 0.5;
			else
			res = ( (_curve[i].elt * 3 - _curve[i+1].elt * 3 + _curve[i+2].elt - _curve[i-1].elt) * h * h * h
				+ (_curve[i-1].elt * 2 + _curve[i+1].elt * 4 - _curve[i+2].elt - _curve[i].elt * 5) * h * h
				+ (_curve[i+1].elt - _curve[i-1].elt) * h
				+ _curve[i].elt * 2) * 0.5;


			return res;
		}

		std::vector<AnimKey>	_curve;
		float		_max_time;
		float		_last_h;
		unsigned int	_last_index;
		bool		_finished;

	};

}

#endif /* _MAINTANCE_CURVE_HH */
