/*
** xml2raytance.cc
** Login : <caf@kasparov>
** Started on  Fri Apr 18 15:52:48 2003 Laurent Corbes {Caf'}
** $Id: xml2raytance.cc,v 1.1.1.1 2004/01/07 23:45:08 Caf Exp $
** 
** Copyright (C) 2003 Laurent Corbes {Caf'}
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <iostream>
#include <fstream>
#include <string>
#include <getopt.h>
#include "loadance/loadance.hh"
#include "config.h"

std::string program_name = "";
std::string xmlfile = "";
std::string outfile = "";

enum exit_codes
  {
    exit_ok = 0,
    loading_error = 1,
    unknown_error = 9
  };

int exit_status = exit_ok;

static void quit_program ()
{
  switch (exit_status)
    {
    case exit_ok:
      break;
    case loading_error:
      std::cerr << program_name << ": loading error" << std::endl;
      break;
    case unknown_error:
    default:
      std::cerr << program_name << ": unknown error" << std::endl;
      break;
    }
  if (exit_status != exit_ok)
    std::cerr << "/!\\ exiting /!\\" << std::endl;
  exit (exit_status);
}

// getopt part
int		verbose = 0;

enum short_options_e
  {
    opt_help = 'h',
    opt_version = 'V',
    opt_verbose = 'v',
  };

static struct option const long_options[] =
  {
    {"help",            no_argument,    0,		opt_help},
    {"version",		no_argument,	0,		opt_version},
    {"verbose",         no_argument,    &verbose,	1},
    {NULL, 0, NULL, 0}
  };

static const char short_options[] = "hvVfwb";

// general fonctions
static void version (void)
{
  std::cerr << PACKAGE << "xml2raytance , " << VERSION << std::endl;
}

static void usage (void)
{
  version();
  std::cerr << "usage: " << program_name << " [-h|-V|-v] "
	    << "xmlfile outpath" << std::endl;
  std::cerr << "..." << std::endl
	    << "-h      --help		       print this message" << std::endl
	    << "-V      --version	       print program version" << std::endl
	    << "-v      --verbose	       print debug messages" << std::endl;
  quit_program ();
}

// main fonction just do the getopt and set the correct runtime options
int main (int argc, char **argv)
{
  program_name = argv[0];
  

  int c;
  while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
         != -1)
    {
      switch (c)
        {
        case opt_help:
          usage();
          break;
	case opt_version:
	  version();
	  quit_program ();
	  break;
	case opt_verbose:
	  verbose = 1;
	  break;
        }
    }
  if (optind != argc - 2)
    usage ();
  
  xmlfile = argv[optind++];
  outfile = argv[optind];
  
  outfile.append("raytance-scene.hh");
  // loading xml file
  loadance::LoadXml xmlscene(xmlfile);
  std::ofstream ostr(outfile.c_str());
  
  ostr << "// Autogenerated raytance-scene.hh output" << std::endl
       << "// do not modify directly use xml2raytance" << std::endl 
       << std::endl;

  ostr << "#include \"base.hh\"" << std::endl
       << "maintance::Scene* load ()" << std::endl
       << "{" << std::endl;
  

  xmlscene.convert(ostr);
  ostr << "};" << std::endl
       << "// end of generated content" << std::endl;


  ostr.close();

  quit_program ();
};
